/* ====
   ==== FlashTris Game, (c) 1999-2001 by FlashCode (Sebastien Helleu)
   ====
   ====    web: http://flashcode.free.fr
   ==== e-mail: flashcode@free.fr
   ====
   ==== game.c: routines du jeu FlashTris
   ====

   ==== This file is part of FlashTris.
   ====
   ==== FlashTris is free software; you can redistribute it and/or modify
   ==== it under the terms of the GNU General Public License as published by
   ==== the Free Software Foundation; either version 2 of the License, or
   ==== (at your option) any later version.
   ====
   ==== FlashTris is distributed in the hope that it will be useful,
   ==== but WITHOUT ANY WARRANTY; without even the implied warranty of
   ==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ==== GNU General Public License for more details.
   ====
   ==== You should have received a copy of the GNU General Public License
   ==== along with FlashTris; if not, write to the Free Software
   ==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@free.fr>
Creation: 1999/06/21

2001/09/06: FlashCode:
    - ajout de la licence
*/


/* VSCR_AUTRES+2: fond avec aires de jeu assombries                                */
/* VSCR_AUTRES+3: utilise pour sauver le fond du bonus a l'initialisation du bonus */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "ftris.h"
#include "key.h"
#include "menu.h"
#include "menu_gfx.h"
#include "video.h"
#include "game.h"
#include "bonus.h"
#include "effet.h"
#include "netftris.h"
#include "./module/mod.h"

#ifdef MSDOS
    #include <dos.h>
    #include <dpmi.h>
    #include <go32.h>
#else
    #include <sys/time.h>
    #include <vgakeyboard.h>
#endif

/*
unsigned char   flashtris[5][36] = { "*** *   *** *** * * *** **  *** ***",
                                     "*   *   * * *   * *  *  * *  *  *  ",
                                     "**  *   *** *** ***  *  **   *  ***",
                                     "*   *   * *   * * *  *  * *  *    *",
                                     "*   *** * * *** * *  *  * * *** ***" };*/



/* codes pour calculer une forme:

-------------------------   -------------------------
|     |     |     |     |   |     |     |     |     |
|32768|16384| 8192| 4096|   | 4096| 256 |  16 |  1  |
|     |     |     |     |   |     |     |     |     |
-------------------------   -------------------------
|     |     |     |     |   |     |     |     |     |
| 2048| 1024| 512 | 256 |   | 8192| 512 |  32 |  2  |
|     |     |     |     |   |     |     |     |     |
-------------------------   -------------------------
|     |     |     |     |   |     |     |     |     |
| 128 |  64 |  32 |  16 |   |16384| 1024|  64 |  4  |
|     |     |     |     |   |     |     |     |     |
-------------------------   -------------------------
|     |     |     |     |   |     |     |     |     |
|  8  |  4  |  2  |  1  |   |32768| 2048| 128 |  8  |
|     |     |     |     |   |     |     |     |     |
-------------------------   -------------------------  */


int             formes[NB_FORMES] =
                { /* *** PIECES STANDARD *** */
                  1024+512+64+32,       /* O */
                  2048+1024+512+256,    /* I */
                  2048+1024+512+64,     /* T */
                  2048+1024+512+128,    /* L */
                  2048+1024+512+32,     /* J */
                  1024+512+128+64,      /* S */
                  2048+1024+64+32,      /* Z */
                  /* *** PIECES RAJOUTEES *** */
                  1024+512+32,          /* petit L */
                  2048+1024+512+
                  256+128+64+32+16,     /* double I */
                  2048+1024+512+64+4,   /* grand T */
                  1024+512+128,         /* idem L sans coin */
                  2048+1024+32,         /* idem J sans coin */
                  512+256+128+64,       /* S long */
                  2048+1024+32+16,      /* Z long */
                  64,                   /* petit carre */
                  2048+1024+512,        /* petite barre (longueur 3) */
                  2048+1024+512+
                  256+64+32,            /* T special */
                  /* *** PIECES BONUS *** */
                  1024+512+64+32 };     /* bombe - NON UTILISE */

int             numero_piece_bonus[NB_FORMES] =
                { 0, 0, 0, 0, 0, 0, 0,          /* les 7 pieces standards     */
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* les 10 pieces eccentriques */
                  PIECE_BOMBE };

int             cptr_niveau[NIVEAU_MAX+1] =     /* vitesse de descente des pieces en fonction du niveau */
                { 58, 53, 48, 43, 38, 33, 28, 23, 18, 13, 8, 7, 6, 5, 4 };

int             X_win_couleur_piece[NB_FORMES] =
                { 4, 1, 6, 2, 14, 3, 5,
                  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ETAT_PIECE_BONUS };

int             couleur_piece[NB_FORMES] =
                { 0, 1, 2, 3, 4, 5, 6,
                  3, 1, 2, 3, 4, 5, 6, 0, 1, 2, ETAT_PIECE_BONUS };

unsigned char   touche[10][10][4] =
                /*{{{SCANCODE_CURSORBLOCKLEFT,SCANCODE_CURSORBLOCKRIGHT,SCANCODE_CURSORBLOCKDOWN,57},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},*/
                {{{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}},
                 {{16,18,31,17},{21,23,36,22},{25,27,40,26},{75,77,80,76},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}}};

int             x_inc[16] = { 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0 };
int             y_inc[16] = { 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0 };

int             couleur_cadre[3] =
                {  150, /* humain */
                   106, /*  ordi  */
                     4  /*  coop  */ };

int             carres_4_lignes[4][4] =
                { {-1,-1,-1, 1 },
                  {-1,-1, 1, 1 },
                  {-1, 1, 1, 1 },
                  { 1, 1, 1, 1 } };

int             carres_3_lignes[3][4] =
                { {-1,-1,-1, 3 },
                  {-1,-1, 3, 3 },
                  {-1, 3, 3, 3 } };

int             carres_2_lignes[2][4] =
                { {-1,-1,-1, 0 },
                  {-1,-1, 0, 0 } };

int             carres_1_ligne[4] =
                {-1,-1,-1, 2 };

int             y_carres_lignes[NBJ_MAX][4];    /* position des compteurs pour     */
                                                /* le nombre de lignes "a la fois" */

int             y_texte_ready[4] = { -20, -10, 0, 12 };

int             coul_texte_ready[4][3] =
                { { 88, 2, 109 },
                  { 88, 2, 109 },
                  { 137, 1, 150 },
                  { 49, 1, 41  } };

t_high_score    high_score[5][10] =
/*    NOM                 NBLIG   NB4 NB3 NB2 NB1   SCORE            SECONDES */
{ { { "Paul Hochon       ", 150, { 10, 12, 20, 34 }, 2560, /*1800,*/ 12*60 },
    { "Tom Hatte         ", 130, {  9, 11, 15, 31 }, 2245, /*1600,*/ 11*60 },
    { "Vincent Tim       ", 110, {  8,  9, 16, 19 }, 1930, /*1400,*/ 9*60+30 },
    { "Paul Position     ",  90, {  7,  8, 12, 14 }, 1620, /*1200,*/ 7*60+30 },
    { "Jean Bambois      ",  80, {  5,  8, 11, 14 }, 1395, /*1100,*/ 6*60+50 },
    { "Clement Thine     ",  70, {  4,  7, 10, 13 }, 1200, /*1000,*/ 6*60 },
    { "Jean Nemarre      ",  60, {  3,  5,  9, 15 },  975, /* 900,*/ 5*60 },
    { "Harry Coverre     ",  50, {  2,  4,  8, 14 },  780, /* 800,*/ 4*60+20 },
    { "Albert Mudat      ",  40, {  2,  3,  7,  9 },  645, /* 700,*/ 3*60+20 },
    { "Yvan Morve        ",  30, {  1,  2,  5, 10 },  445, /* 600,*/ 2*60+30 } },

  { { "Paul Hochon       ",  16, {  1,  1,  2,  5 },  260, /* 190,*/ 0 },
    { "Tom Hatte         ",  15, {  1,  1,  1,  6 },  245, /* 180,*/ 0 },
    { "Vincent Tim       ",  14, {  0,  2,  2,  4 },  210, /* 170,*/ 0 },
    { "Paul Position     ",  13, {  0,  1,  2,  6 },  170, /* 160,*/ 0 },
    { "Jean Bambois      ",  12, {  0,  1,  2,  5 },  160, /* 150,*/ 0 },
    { "Clement Thine     ",  11, {  0,  0,  3,  5 },  125, /* 140,*/ 0 },
    { "Jean Nemarre      ",  10, {  0,  0,  3,  4 },  115, /* 130,*/ 0 },
    { "Harry Coverre     ",   9, {  0,  0,  2,  5 },  100, /* 120,*/ 0 },
    { "Albert Mudat      ",   8, {  0,  0,  2,  4 },   90, /* 110,*/ 0 },
    { "Yvan Morve        ",   7, {  0,  0,  1,  5 },   75, /* 100,*/ 0 } },

  { { "Paul Hochon       ",  32, {  1,  3,  4, 11 },  490, /* 380,*/ 0 },
    { "Tom Hatte         ",  29, {  1,  3,  4,  8 },  460, /* 350,*/ 0 },
    { "Vincent Tim       ",  28, {  0,  2,  7,  8 },  375, /* 340,*/ 0 },
    { "Paul Position     ",  27, {  0,  2,  6,  9 },  360, /* 330,*/ 0 },
    { "Jean Bambois      ",  26, {  0,  2,  5, 10 },  345, /* 320,*/ 0 },
    { "Clement Thine     ",  25, {  0,  2,  5,  9 },  335, /* 310,*/ 0 },
    { "Jean Nemarre      ",  24, {  0,  1,  6,  9 },  300, /* 300,*/ 0 },
    { "Harry Coverre     ",  22, {  0,  1,  5,  9 },  275, /* 280,*/ 0 },
    { "Albert Mudat      ",  21, {  0,  1,  4, 10 },  260, /* 270,*/ 0 },
    { "Yvan Morve        ",  20, {  0,  0,  5, 10 },  225, /* 260,*/ 0 } },

  { { "Paul Hochon       ",  80, {  3,  8, 11, 22 }, 1275, /* 950,*/ 0 },
    { "Tom Hatte         ",  75, {  3,  7, 10, 22 }, 1190, /* 900,*/ 0 },
    { "Vincent Tim       ",  72, {  2,  7, 10, 23 }, 1100, /* 870,*/ 0 },
    { "Paul Position     ",  70, {  2,  7,  9, 23 }, 1075, /* 850,*/ 0 },
    { "Jean Bambois      ",  67, {  2,  6, 10, 21 }, 1020, /* 820,*/ 0 },
    { "Clement Thine     ",  65, {  2,  6,  9, 21 },  995, /* 790,*/ 0 },
    { "Jean Nemarre      ",  62, {  1,  6, 10, 20 },  910, /* 760,*/ 0 },
    { "Harry Coverre     ",  60, {  1,  6,  9, 20 },  885, /* 740,*/ 0 },
    { "Albert Mudat      ",  55, {  1,  5,  8, 20 },  800, /* 690,*/ 0 },
    { "Yvan Morve        ",  50, {  0,  7,  7, 15 },  745, /* 640,*/ 0 } },

  { { "Paul Hochon       ", 140, {  8, 11, 17, 41 }, 2295, /*1600,*/ 0 },
    { "Tom Hatte         ", 130, {  7, 10, 15, 42 }, 2095, /*1500,*/ 0 },
    { "Vincent Tim       ", 110, {  7,  9, 14, 27 }, 1860, /*1300,*/ 0 },
    { "Paul Position     ",  90, {  6,  8, 13, 16 }, 1565, /*1000,*/ 0 },
    { "Jean Bambois      ",  80, {  5,  6, 10, 22 }, 1330, /* 900,*/ 0 },
    { "Clement Thine     ",  70, {  4,  5,  9, 21 }, 1135, /* 800,*/ 0 },
    { "Jean Nemarre      ",  60, {  3,  4,  8, 20 },  940, /* 700,*/ 0 },
    { "Harry Coverre     ",  50, {  2,  4,  7, 16 },  775, /* 600,*/ 0 },
    { "Albert Mudat      ",  45, {  1,  3,  7, 18 },  635, /* 550,*/ 0 },
    { "Yvan Morve        ",  40, {  0,  3,  6, 19 },  520, /* 500,*/ 0 } } };

t_challenge     challenge[NIVEAU_MAX+1] =
    { /* 0 */{ 5, 0 }, /* 1 */{ 10, 0 }, /* 2 */{ 20, 0 }, /* 3 */{ 5, 2*60 },
      /* 4 */{ 10, 3*60+45 }, /* 5 */{ 20, 7*60 }, /* 6 */{ 5, 1*60+40 },
      /* 7 */{ 10, 3*60 }, /* 8 */{ 20, 5*60 }, /* 9 */{ 5, 1*60 },
      /* 10 */{ 10, 1*60+45 }, /* 11 */{ 20, 3*60 }, /* 12 */{ 5, 0*60+35 },
      /* 13 */{ 10, 0*60+45 }, /* 14 */{ 20, 0*60+50 } };


/* ================ JEU ================ */

int             game_mode;              /* type de jeu: classique, challenge, "star"                             */
int             game_team;              /* cooperatif ?                                                          */
int             nbp;                    /* nombre de joueurs humains                                             */
int             nbc;                    /* nombre de joueurs geres par l'ordinateur                              */
int             nbj;                    /* nombre de joueurs au total                                            */
int             nbx;                    /* nombre de carres horizontalement dans chaque aire de jeu (defaut: 10) */
int             nby;                    /* nombre de carres verticalement dans chaque aire de jeu (defaut: 20)   */
int             ordi[NBJ_MAX];          /* type de joueur: 1=ordi 0=humain                                       */
int             nb_PC_net;              /* nombre de PC connectes                                                */
t_PC_net        PC_net[NET_NBJ_MAX];    /* infos reseau sur chaque PC connecte                                   */
int             numero_PC_net;          /* 0 pour le serveur ou > 0 pour un client (recu du serveur dans le      */
                                        /* msg 'Start game')                                                     */
int             perdu[NBJ_MAX];         /* 1 si le joueur/equipe a perdu (0 s'il joue)                           */
int             tous_perdu;             /* 1 si tous les joueurs ont perdu                                       */
int             partie_finie;           /* 1 si la partie est finie (tous perdu ou un seul joueur/equipe reste)  */
int             bonus_prevu[NBJ_MAX];   /* numero du bonus prevu (pour attendre la fin de l'explosion avant de   */
                                        /* rentrer dans le niveau bonus)                                         */
int             bonus[NBJ_MAX];         /* numero du niveau bonus en cours (0 = pas de niveau bonus en cours)    */
int             nb_lignes[NBJ_MAX][5];  /* nombre de lignes eliminees depuis le debut :                          */
                                        /* [0] = total                                                           */
                                        /* [1] = nombre de fois 1 ligne                                          */
                                        /* [2] = nombre de fois 2 lignes                                         */
                                        /* [3] = nombre de fois 3 lignes                                         */
                                        /* [4] = nombre de FlashTris !!!!!!                                      */
int             nb_posees[NBJ_MAX][7];  /* nombre de pieces posees par joueur et par piece                       */
int             nb_explosions[NBJ_MAX]; /* nombre de lignes qui explosent (>=0)                                  */
int             explosions[NBJ_MAX][ETAT_Y_MAX];
                                        /* numero des lignes qui explosent                                       */
int             aff_explosions[NBJ_MAX];/* >0 si l'explosion s'affiche (pour attendre 4 waitretraces)            */

int             descend_cptr[NBJ_MAX];  /* compteur pour vitesse de descente                                     */
int             descend[NBJ_MAX];       /* = 1 si la piece doit descendre                                        */
int             touche_cptr[NBJ_MAX];   /* pour ralentir le deplacement par le joueur                            */
int             rotation_cptr[NBJ_MAX]; /* pour ralentir la rotation                                             */

int             challenge_ready_affiche;/* 1 si l'objectif du niveau challenge est affiche                       */
int             challenge_timer_ready;  /* delai d'affichage de l'objectif du niveau en mode challenge           */
int             challenge_clignote_lignes;
                                        /* compteur pour faire clignoter le nombre de lignes restant a faire     */

int             envoye[NBJ_MAX];        /* nombre de lignes au total envoyees par chaque joueur                  */
int             bascule_envoi[NBJ_MAX]; /* bascule d'envoi d'une ligne                                           */
int             doit_remonter[NBJ_MAX]; /* nombre de lignes que le joueur va recevoir                            */

signed char     etat[ETAT_Y_MAX][ETAT_X_MAX];       /* etat des cases                                            */
signed char     etat_pause[ETAT_Y_MAX][ETAT_X_MAX]; /* etats sauves pendant la pause                             */

int             x_deb[NBJ_MAX], x_fin[NBJ_MAX];     /* debut de l'aire de jeu X (en pixels)                      */
int             y_deb[NBJ_MAX], y_fin[NBJ_MAX];     /* debut de l'aire de jeu Y (en pixels)                      */
int             x_next[NBJ_MAX], y_next[NBJ_MAX];   /* debut de l'aire de la piece suivante (en pixels)          */
int             x_next2[NBJ_MAX], y_next2[NBJ_MAX]; /* debut de l'aire de la 2eme piece suivante (en pixels)     */

int             x_min[NBJ_MAX], x_max[NBJ_MAX];     /* mini et maxi X (en coordonnees dans etat)                 */
int             y_min[NBJ_MAX], y_max[NBJ_MAX];     /* mini et maxi Y (en coordonnees dans etat)                 */

int             x_time, y_time;             /* coordonnees du "time"                                             */

t_piece         piece[NBJ_MAX];             /* pieces des joueurs                                                */
t_piece         piece_pause[NBJ_MAX];       /* pieces sauvees pendant la pause                                   */
int             next_forme[NBJ_MAX];        /* forme de la piece suivante                                        */
int             next_forme_pause[NBJ_MAX];  /* forme suivanre sauvee pendant la pause                            */
int             next_forme2[NBJ_MAX];       /* forme 2eme piece suivante                                         */
int             next_forme2_pause[NBJ_MAX]; /* forme 2eme piece sauvee pendant la pause                          */
t_piece         next_piece[NBJ_MAX];        /* piece suivante pour chaque joueur                                 */
t_piece         next_piece_pause[NBJ_MAX];  /* piece suivante sauvee pendant la pause                            */
t_piece         next_piece2[NBJ_MAX];       /* 2eme piece suivante pour chaque joueur                            */
t_piece         next_piece2_pause[NBJ_MAX]; /* 2eme piece suivante sauvee pendant la pause                       */
t_piece         piece_old[NBJ_MAX];         /* pour effacement des pieces (sauve ici les anciennes coordonnees   */
                                            /* et l'ancienne forme)                                              */
t_piece         piece_old_pause[NBJ_MAX];   /* pour effacement des pieces sauvee pendant la pause                */

int             forme_avance[NB_FORMES_AV];         /* pieces tirees d'avance (utilise si same_pieces = 1)       */
int             forme_avance_pause[NB_FORMES_AV];   /* pieces tirees d'avance sauvees pendant la pause           */
int             numero_forme[NBJ_MAX];              /* numero de la piece du joueur                              */
int             numero_forme_pause[NBJ_MAX];        /* numero de la piece du joueur sauvee pendant la pause      */

int             bonne_colonne[NBJ_MAX]; /* colonne choisie par l'ordinateur                                      */
int             bonne_forme[NBJ_MAX];   /* orientation choisie par l'ordinateur                                  */

int             difficulte;     /* difficulte (rapidite de l'ordinateur)                                         */
int             same_pieces;    /* tous les joueurs recoivent les memes pieces                                   */
int             eccentric;      /* pieces bizarres ? 1=oui 0=non                                                 */
int             hauteur_ini;    /* hauteur initiale                                                              */
int             time_limite;    /* temps limite                                                                  */
int             cptr_time;      /* compteur de temps (augmente si time_limite = 0, sinon diminue)                */
int             time_change;    /* = 1 quand il faut reafficher le temps                                         */
int             view_next;      /* piece suivante affichee ?                                                     */
int             stats;          /* statistiques sur les pieces tombees                                           */
int             acceler;        /* acceleration (direct en bas, rapide, lent, ou tres lent)                      */
int             tc;             /* taille des carres des pieces                                                  */
int             tc_stats;       /* taille des carres pour les pieces des stats                                   */
int             tc_next;        /* taille des carres pour la piece suivante                                      */
int             tc_next2;       /* taille des carres pour la 2eme piece suivante                                 */
int             fond_jeu;       /* fond du jeu (numero de l'image dans *nom_fond                                 */
int             bonus_level;    /* niveaux bonus dans le jeu                                                     */
int             status_jeu;     /* statut du jeu en bas de l'ecran ?                                             */
int             window_msg;     /* fenetre de messages en dessous de chaque joueur ?                             */
unsigned char   msg[NBJ_MAX][2][50];    /* les 2 messages de chaque joueur                                       */
int             cptr_msg[NBJ_MAX];      /* pour faire disparaitre les messages au bout d'un moment               */
int             cheat;
int             aire_de_jeu;    /* aire de jeu (0=normale 1=grande)                                              */
int             nbj_team[3];    /* nombre de joueurs dans chaque equipe                                          */
int             team[NBJ_MAX];  /* numero de l'equipe de chaque joueur                                           */

int             en_jeu;         /* = 1 si une partie est en cours                                                */
int             en_pause;       /* = 1 si le jeu est en pause                                                    */
int             quitte_jeu;     /* = 1 si le joueur veut quitter le jeu                                          */

int             niveau;

int             ecran_inverse[NBJ_MAX]; /* ecran inverse ?? !!                                                   */

int             image_bombe;            /* pour afficher les 2 images de bombes                                  */

int             x_volume_jeu;           /* coordonnees pour l'affichage du volume en bas de l'ecran              */
int             x_musique_jeu;          /* idem pour le nom de la musique                                        */
int             x_barre_musique;        /* idem pour la barre d'avancement de la musique                         */

int             coul_winner;            /* couleur du mot "Winner"                                               */
int             dir_coul_winner;        /* +1 ou -1                                                              */


/* =============== TIMER =============== */

#ifdef MSDOS
int                 dos_nb_appels;
int                 dos_dans_timer;
__dpmi_paddr        old_int08;
__dpmi_raddr        old_int71_real;
_go32_dpmi_seginfo  wrap_int08;
int                 compteur_usleep_dos;
#else
long            linux_timer1_sec;
long            linux_timer1_usec;
#endif
int             nb_appels;


#ifdef MSDOS
    #define CLI __asm__ __volatile__ ("cli");
    #define STI __asm__ __volatile__ ("sti");
#endif


/*void afflog(int type_log, unsigned char *texte)
{
    unsigned char   buffer[100];

    if (type_log == 0)
        niveau_log--;
    memset(buffer, ' ', 100);
    buffer[niveau_log*2] = '\0';
    sprintf(&buffer[strlen(buffer)], "%s %s", (type_log == 1)?"**":"--", texte);
    fprintf(log, buffer);
    if (type_log == 1)
        niveau_log++;
}*/

/*
 * get_game_team: renvoie 1 si jeu en equipe et possible, 0 sinon
 */

int get_game_team()
{
    if (nbc+nbp <= 1)
        return 0;
    else
        return game_team;
}

/*
 * get_hauteur_ini: renvoie le handicap si possible, 0 sinon
 */

int get_hauteur_ini()
{
    if (network)
        return 0;
    else
        return hauteur_ini;
}

/*
 * get_stats: renvoie 1 s'il y a les stats, 0 sinon
 */

int get_stats()
{
    if (nbp+nbc > 3)
        return 0;
    else
        return stats;
}

/*
 * get_time_limite: renvoie le temps limite + 1 si possible, 0 sinon
 */

int get_time_limite()
{
    if ( (en_jeu) && (game_mode == GAME_CHALLENGE) )
        return (challenge[niveau].c_temps_limite != 0)+1;
    if (network)
        return 0;
    else
        return time_limite+1;
}

/*
 * get_bonus_level: renvoie 1 s'il y a les niveaux bonus et possible, 0 sinon
 */

int get_bonus_level()
{
    /*  et par reseau ???!!!  */
    if ( (network) || (game_mode == GAME_CHALLENGE) || (nbc > 0) || (nbp > 4)
        || (get_game_team()) || (get_time_limite() > 1) )
        return 0;
    else
        return bonus_level;
}

/*
 * get_window_msg: renvoie 1 s'il y a les fenetres de messages et possible, 0 sinon
 */

int get_window_msg()
{
    if (nbp+nbc > 4)
        return 0;
    else
        return window_msg;
}

/*
 * get_aire_de_jeu: renvoie l'aire de jeu
 */

int get_aire_de_jeu()
{
    if (game_mode == GAME_CHALLENGE)
        return AIRE_NORMALE;
    else
        return aire_de_jeu;
}

/* ==========================================================================

                                     TIMER

   ========================================================================== */

/*
 * linux_init_timer: initialise le timer pour Linux
 */

#ifndef MSDOS
void linux_init_timer()
{
    struct timeval  tv;
    struct timezone tz;

    gettimeofday(&tv, &tz);
    linux_timer1_sec = tv.tv_sec;
    linux_timer1_usec = tv.tv_usec;
}
#endif

/*
 * nouveau_timer: nouveau timer pour FlashTris (pour faire tomber les pieces et
 *                compter les secondes)
 */

void nouveau_timer()
{
    #ifdef MSDOS
    __dpmi_regs     regs;
    #else
    struct timeval  tv;
    struct timezone tz;
    long            intervalle;
    #endif
    int             i, limite;

    #ifdef MSDOS
    if (dos_dans_timer)
        ftris_error(MSG_ERROR, BUG, CLOSE_SB16, STOP_PROG_GRAVE,
            "timer reentrance");
    dos_dans_timer = 1;
    compteur_usleep_dos = (compteur_usleep_dos+1)%1000000000;
    #endif
    /*  manque des choses  */
    #ifdef MSDOS
    dos_nb_appels++;
    if (dos_nb_appels == 5)
    {
        dos_nb_appels = 0;
        /* appel de l'ancienne interruption 0x08 pour faire avancer l'horloge ! */
        __dpmi_simulate_real_mode_interrupt(0x71, &regs);
    }
    #endif

    #ifndef MSDOS
    gettimeofday(&tv, &tz);
    /*DEBUG:*/
    /*printf("* timer1_sec=%ld  timer1_usec=%ld\n", linux_timer1_sec, linux_timer1_usec);*/
    /*printf("  timer_sec =%ld  timer_usec =%ld\n", tv.tv_sec, tv.tv_usec);*/
    if (tv.tv_usec < linux_timer1_usec)
        intervalle = (tv.tv_sec-linux_timer1_sec)*1000000+
            (1000000-linux_timer1_usec)+tv.tv_usec;
    else
        intervalle = (tv.tv_sec-linux_timer1_sec)*1000000+
            (tv.tv_usec-linux_timer1_usec);
    /*DEBUG:*/
    /*printf("  intervalle=%ld\n", intervalle);*/
    if (intervalle >= 10989)    /* 10989 = 1000000 / 91 */
    {
        if (linux_timer1_usec >= 989011)
            linux_timer1_sec++;
        linux_timer1_usec = (linux_timer1_usec+10989)%1000000;
    #endif
        nb_appels++;
        if (challenge_timer_ready > 0)
            challenge_timer_ready--;
        if (nb_appels == 91)
        {
            nb_appels = 0;
            /*  decrementation secondes .... pour timer jeu .... */
            if ( (en_jeu) && (!en_pause) && (!partie_finie) )
            {
                if (!challenge_ready_affiche)
                {
                    switch (get_time_limite())
                    {
                        case 0:
                            break;
                        case 1:
                            cptr_time++;
                            time_change = 1;
                            break;
                        default:
                            cptr_time--;
                            if (cptr_time < 0)
                            {
                                cptr_time = 0;
                                if (!tous_perdu)
                                    for (i = 0; i < nbj; i++)
                                        if ( (i == 0) || (team[i] != team[i-1]) )
                                            joueur_ou_team_perd(i);
                                tous_perdu = 1;
                                partie_finie = 1;
                            }
                            else
                                time_change = 1;
                    }
                }
            }
        }
        if ( (en_jeu) && (!en_pause) )
        {
            for (i = 0; i < nbj; i++)
                if (ordi[i])
                {
                    descend_cptr[i]++;
                    switch (difficulte)
                    {
                        case 0:
                            if (descend_cptr[i] >= 58-5*niveau)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                        case 1:
                            if (descend_cptr[i] >= 38-3*niveau)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                        case 2:
                            if (descend_cptr[i] >= 18-niveau)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                        case 3:
                            if (descend_cptr[i] >= 11-niveau)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                        case 4:
                            if (descend_cptr[i] >= 4)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                        case 5:
                            if (descend_cptr[i] >= 2)
                                { descend_cptr[i] = 0; descend[i] = 1; }
                            break;
                    }
                    if (rotation_cptr[i] > 0) rotation_cptr[i]--;
                }
                else
                    if (bonus[i] == NO_BONUS)
                    {
                        descend_cptr[i]++;
                        if (game_mode == GAME_CHALLENGE)
                            limite = cptr_niveau[(niveau*2)/3];
                        else
                            limite = cptr_niveau[niveau];
                        if (descend_cptr[i] >= limite)
                        {
                            descend_cptr[i] = 0;
                            descend[i] = 1;
                        }
                        if (touche_cptr[i] > 0) touche_cptr[i]--;       /* ralentissement deplacement */
                        if (rotation_cptr[i] > 0) rotation_cptr[i]--;   /* ralentissement rotation    */
                    }
                    else
                    {
                        if (bonus_timer_ready[i] > 0)
                            bonus_timer_ready[i]--;
                        else
                        {
                            bonus_nb_appels[i]++;
                            if (bonus_nb_appels[i] == 91)
                            {
                                bonus_nb_appels[i] = 0;
                                if (bonus_sec_reste[i] > 0)
                                {
                                    bonus_sec_reste[i]--;
                                    bonus_sec_change[i] = 1;
                                }
                            }
                        }
                    }
        }
        #ifdef MSDOS
        if ( (sb16) && (playing_module) )
            sbpoll();
        dos_dans_timer = 0;
        outportb(0x20, 0x20);
        #else
        /*signal(SIGALRM, nouveau_timer);*/
        #endif
    #ifndef MSDOS
    }
    #endif
}

#ifdef MSDOS
/* Version Dos : */
void detourne_int_timer()
{
    int             i;
    __dpmi_raddr    old_int08_real;

    /* nouvelle interruption 08: 91 fois par seconde (18.2 * 5) */
    CLI
    i = 1193180/91;
    outportb(0x43, 0x36);
    outportb(0x40, (i & 0xFF));
    outportb(0x40, (i >> 8));

    /* int71(r) <-- int08(r) */
    __dpmi_get_real_mode_interrupt_vector(0x08, &old_int08_real);
    __dpmi_get_real_mode_interrupt_vector(0x71, &old_int71_real);
    __dpmi_set_real_mode_interrupt_vector(0x71, &old_int08_real);

    /* int08(p) <-- nouveau_timer */
    nb_appels = 0;
    dos_dans_timer = 0;
    detourne_interruption(0x08, (int)nouveau_timer, &old_int08, &wrap_int08);
    STI
}
#else
/* Version Linux : */
void detourne_int_timer()
{
    linux_init_timer();     /* c'est tout ! */
}
#endif

#ifdef MSDOS
/* Version Dos : */
void restaure_int_timer()
{
    CLI
    outportb(0x43, 0x36);
    outportb(0x40, 0);
    outportb(0x40, 0);
    restaure_interruption(0x08, &old_int08, &wrap_int08);
    __dpmi_set_real_mode_interrupt_vector(0x71, &old_int71_real);
    STI
}
#else
/* Version Linux : */
void restaure_int_timer()
{
    /* rien a faire !!! */
}
#endif

/* ==========================================================================

                          ROUTINES D'AFFICHAGE POUR LE JEU

   ========================================================================== */

/*
 * calcule_pos_joueurs: calcule la position des joueurs en fonction du
 *                      nombre de joueurs et des options de jeu (stats,...)
 */

void calcule_pos_joueurs()
{
    int     i, j, ajout_x, ajout_x_2, ajout_y, ajout_y_dessus;
    int     nb_lig, nbj_par_ligne, tab_nbj_par_ligne[10];
    int     espace_x, espace_y, taille_x, taille_y;
    int     nbx_dispo, nby_dispo;
    int     x, y;

    nbj = nbp+nbc;

    nbx_dispo = scr_nbx;
    nby_dispo = (status_jeu)?scr_nby-6:scr_nby;

    if (get_game_team())
    {
        for (i = 0; i < nbj; i++)
            /*  il peut y avoir plusieurs equipes !!! */
            team[i] = 0;
    }
    else
    {
        for (i = 0; i < nbj; i++)
            team[i] = i;
    }

    /*  calcul automatique de la taille des carres */
    tc = 0;
    if (get_game_team())
    {
        if (get_aire_de_jeu() == AIRE_LARGE)
        {
            nbx = 20+4*nbj;
            nby = (nbj > 2)?25+(nbj-2)*2:25;
        }
        else
        {
            nbx = 10+4*nbj;
            nby = (nbj > 2)?20+(nbj-2)*2:20;
        }
    }
    else
    {
        if (get_aire_de_jeu() == AIRE_LARGE)
        {
            nbx = 20;
            nby = 26;
        }
        else
        {
            nbx = 10;
            nby = 20;
        }
    }
    for (i = TC_MAX; i >= 2; i--)
    {
        ajout_x = 0;

        if (get_stats())
        {
            if (i <= 6)
                tc_stats = i/2;
            else
                tc_stats = 3;
            ajout_x = 4+tc_stats*4+3;
        }
        if (ajout_x < 23)
            ajout_x = 23;

        ajout_y = 0;

        if (get_window_msg())
            ajout_y += 17;

        if (get_game_team())
        {
            ajout_y_dessus = 0;
            if (i <= 4)
                    tc_next = i;
                else
                    tc_next = 4;
                tc_next2 = (tc_next > TC_MIN+1)?tc_next-2:TC_MIN;
            if (view_next > 0)
            {
                ajout_y += (4*tc_next)+8;
                ajout_y_dessus += (4*tc_next)+8;
                if (view_next == 2)
                {
                    ajout_y += (4*tc_next2)+7;
                    ajout_y_dessus += (4*tc_next2)+7;
                }
            }
        }
        else
        {
            ajout_y_dessus = 0; /* pour eviter un warning en compilant avec -O2 */
            if (view_next > 0)
            {
                if (i <= 4)
                    tc_next = i;
                else
                    tc_next = 4;
                ajout_x_2 = 2+tc_next*4+2;
            }
            else
                ajout_x_2 = 0;
            if (ajout_x_2 > ajout_x) ajout_x = ajout_x_2;

            if (tc_next > TC_MIN+1)
                tc_next2 = tc_next-2;
            else
                tc_next2 = TC_MIN;
        }

        taille_x = nbx*i+ajout_x+5;/*4;*/
        taille_y = nby*i+ajout_y+2;
        if (get_game_team())
        {
            nb_lig = 0;     /* pour eviter un warning en compilant avec -O2 */
            if ( (taille_x < nbx_dispo) && (taille_y < nby_dispo) )
            {
                tc = i;
                break;
            }
        }
        else
        {
            nb_lig = nby_dispo/taille_y;
            nbj_par_ligne = nbx_dispo/taille_x;
            if (nb_lig*nbj_par_ligne >= nbj)
            {
                tc = i;
                break;
            }
        }
    }
    if (tc == 0)
        ftris_error(MSG_ERROR, BUG, CLOSE_SB16, STOP_PROG,
            "unable to calculate square size");

    if (get_game_team())
    {
        /*  gerer plusieurs equipes !!! */
        ordi[0]  = (nbp == 0);
        x_min[0] = 3;
        x_max[0] = x_min[0]+nbx-1;
        y_min[0] = 3;
        y_max[0] = y_min[0]+nby-1;
        x_deb[0] = (nbx_dispo-taille_x)/2;
        x_fin[0] = x_deb[0]+(x_max[0]-x_min[0]+1)*tc;
        y_deb[0] = (nby_dispo-taille_y)/2+ajout_y_dessus;
        y_fin[0] = y_deb[0]+(y_max[0]-y_min[0]+1)*tc;
        x_next[0] = x_deb[0]+4*tc;
        y_next[0] = y_deb[0]-4*tc_next-8;
        x_next2[0] = x_next[0]+(4*tc_next-4*tc_next2)/2;
        y_next2[0] = y_next[0]-4*tc_next2-7;
        for (i = 1; i < nbj; i++)
        {
            ordi[i]  = (i >= nbp);
            x_min[i] = x_min[0];
            x_max[i] = x_max[0];
            y_min[i] = y_min[0];
            y_max[i] = y_max[0];
            x_deb[i] = x_deb[0];
            x_fin[i] = x_fin[0];
            y_deb[i] = y_deb[0];
            y_fin[i] = y_fin[0];
            /*  attention pour plusieurs equipes ("*i" !!) */
            x_next[i] = x_deb[0]+
                (((x_max[0]-x_min[0]-4*nbj)/(nbj+1)+4)*i+4)*tc;
            y_next[i] = y_deb[0]-4*tc_next-8;
            x_next2[i] = x_next[i]+(4*tc_next-4*tc_next2)/2;
            y_next2[i] = y_next[i]-4*tc_next2-7;
        }
    }
    else
    {
        nbj_par_ligne = nbj/nb_lig;
        if (nbj%nb_lig != 0)
            nbj_par_ligne++;

        j = nbj;
        for (i = 0; i < nb_lig; i++)
            if (j >= nbj_par_ligne)
            {
                tab_nbj_par_ligne[i] = nbj_par_ligne;
                j -= nbj_par_ligne;
            }
            else
            {
                tab_nbj_par_ligne[i] = j;
                j = 0;
            }

        /* calcul de la position de chaque joueur en fonction de la taille des carres */
        for (i = 0; i < nbj; i++)
        {
            ordi[i] = (i >= nbp);

            espace_x = (nbx_dispo-(nbj_par_ligne*taille_x))/(nbj_par_ligne+1);
            espace_y = (nby_dispo-(nb_lig*taille_y))/(nb_lig+1);

            x = i%nbj_par_ligne;
            y = i/nbj_par_ligne;

            x_min[i] = i*(nbx+5);
            x_max[i] = x_min[i]+nbx-1;

            y_min[i] = 3;
            y_max[i] = y_min[i]+nby-1;

            x_deb[i] = x*(taille_x+espace_x)+espace_x;
            x_fin[i] = x_deb[i]+(x_max[i]-x_min[i]+1)*tc;

            y_deb[i] = y*(taille_y+espace_y)+espace_y;
            y_fin[i] = y_deb[i]+(y_max[i]-y_min[i]+1)*tc;

            x_next[i] = x_fin[i]+2;
            y_next[i] = y_deb[i]-1;
            x_next2[i] = x_fin[i]+2;
            y_next2[i] = y_deb[i]-1+4*tc_next+7;
        }
    }

    if (get_game_team())
        ajout_y = 0;
    else
        switch (view_next)
        {
            case 0:ajout_y = -1; break;
            case 1:ajout_y = 4*tc_next+6; break;
            case 2:ajout_y = 4*tc_next+7+4*tc_next2+7; break;
        }

    for (i = 0; i < nbj; i++)
        if ( (i == 0) || (team[i] != team[i-1]) )
        {
            y_carres_lignes[i][3] = y_deb[i]+ajout_y;
            y_carres_lignes[i][2] = y_deb[i]+ajout_y+4*tc_next2+2;
            y_carres_lignes[i][1] = y_deb[i]+ajout_y+7*tc_next2+4;
            y_carres_lignes[i][0] = y_deb[i]+ajout_y+9*tc_next2+7;
        }

    /*printf("calcule_pos_joueurs; game_mode = %d  nbx=%d  nby=%d\n", game_mode, nbx, nby);
    for (i = 0; i < nbj; i++)
        printf("j%d: x1=%d x2=%d y1=%d y2=%d xdeb=%d xfin=%d ydeb=%d yfin=%d\n",
            i, x_min[i], x_max[i], y_min[i], y_max[i], x_deb[i], x_fin[i],
            y_deb[i], y_fin[i]);*/
    if ( (nbj == 1) || (get_game_team()) )
    {
        x_time = x_fin[0]+4;
        y_time = y_carres_lignes[0][0]+9+9;
    }
    else
    {
        x_time = scr_nbx-20;
        y_time = 2;
    }
}

/*
 * affiche_volume_jeu: affiche le volume en bas de l'ecran
 */

void affiche_volume_jeu(int no_vscr)
{
    unsigned char   texte[10], chaine[10];
    int             i, j;

    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_VSCREEN, x_volume_jeu, 194, 12, 5);
    else
        copy_v2v_xy(VSCR_VSCREEN, no_vscr, x_volume_jeu, 194, 12, 5);
    sprintf(texte, "%3d", (volume*100)/VOLUME_MAX);
    chaine[1] = '\0';
    for (i = 0; i < 3; i++)
    {
        chaine[0] = texte[i];
        j = (chaine[0] == '1')?1:0;
        write_xy_s_c(no_vscr, x_volume_jeu+(i*4)+j, 194, 73, 1, chaine);
    }
}

/*
 * affiche_musique_jeu: affiche le nom de la musique en bas de l'ecran
 */

void affiche_musique_jeu(int no_vscr)
{
    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_VSCREEN, x_musique_jeu, 194, scr_nbx-x_musique_jeu, 5);
    else
        copy_v2v_xy(VSCR_VSCREEN, no_vscr, x_musique_jeu, 194, scr_nbx-x_musique_jeu, 5);
    write_xy_s_c(no_vscr, x_musique_jeu, 194, 73, 1, get_lib_music(music_jeu));
    x_barre_musique = x_musique_jeu+text_length(get_lib_music(music_jeu))+5;

    if (music_jeu != NO_MUSIC)
    {
        /*rectangle(no_vscr, x_barre_musique, 194, x_barre_musique+101, 197, 126);*/
        ligne_horiz(no_vscr, x_barre_musique, x_barre_musique+2, 194, 104);
        ligne_horiz(no_vscr, x_barre_musique, x_barre_musique+2, 197, 104);
        ligne_vertic(no_vscr, x_barre_musique, 195, 196, 104);
        ligne_horiz(no_vscr, x_barre_musique+101-2, x_barre_musique+101, 194, 104);
        ligne_horiz(no_vscr, x_barre_musique+101-2, x_barre_musique+101, 197, 104);
        ligne_vertic(no_vscr, x_barre_musique+101, 195, 196, 104);
    }
}

/*
 * game_affiche_pos_musique: affiche la barre d'avancement dans la musique
 */

void game_affiche_pos_musique(int no_vscr, int force_affichage)
{
    int             i, nb_lignes;

    if ( (sb16) && (status_jeu) && (music_jeu != NO_MUSIC) && (row != 64) )
    {
        if (orderpos <= 0)
            nb_lignes = 0;
        else
            nb_lignes = ((((orderpos-1)*64)+row)*100)/((orderlen*64)-1);
        if ( (nb_lignes != pos_musique) || (force_affichage) )
        {
            for (i = 0; i < nb_lignes; i++)
            {
                /*ligne_vertic(no_vscr, x_barre_musique+i+1, 195, 196, 123);*/
                putpixel_ofs(no_vscr, x_barre_musique+i+1+(195*scr_nbx), 123);
                putpixel_ofs(no_vscr, x_barre_musique+i+1+(196*scr_nbx), 124);
            }
            if ( (nb_lignes < 100) && (nb_lignes < pos_musique) )
            {
                if (no_vscr == ECRAN_REEL)
                    copy_v2s_xy(VSCR_JEU_VIDE, x_barre_musique+nb_lignes+1, 195, 100-nb_lignes, 2);
                else
                    copy_v2v_xy(VSCR_JEU_VIDE, no_vscr, x_barre_musique+nb_lignes+1, 195, 100-nb_lignes, 2);
            }
            pos_musique = nb_lignes;
        }
    }
}

/*
 * dessine_image_jeu: dessine l'image de jeu (fond + aires de jeu)
 *                    scr: image de destination
 *                    scr_fond: image contenant le fond d'ecran du jeu
 *                    cadres: 1 si dessin des cadres des aires de jeu, 0 sinon
 */

void dessine_image_jeu(int no_vscr, int no_vscr_fond, int no_vscr_fond_sombre,
                       int assombrit, int cadres)
{
    int     i, j, k;

    copy_v2v(no_vscr_fond, no_vscr);

    if (cadres || assombrit)
    {
        if (get_game_team())
        {
            /*  gerer plusieurs equipes !!! */
            if (cadres)
            {
                rectangle(no_vscr, x_deb[0]-1, y_deb[0]-1,
                    x_deb[0]+(x_max[0]-x_min[0]+1)*tc,
                    y_deb[0]+(y_max[0]-y_min[0]+1)*tc, couleur_cadre[2]);
                if (get_window_msg())
                    rectangle(no_vscr, x_deb[0]-1, y_fin[0]+3,
                        x_fin[0], y_fin[0]+3+14, couleur_cadre[2]);
            }
            if (assombrit)
            {
                copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_deb[0], y_deb[0],
                    x_fin[0]-x_deb[0], y_fin[0]-y_deb[0]);
                if (get_window_msg())
                    copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_deb[0],
                        y_fin[0]+4, x_fin[0]-x_deb[0], 13);
            }
            if (get_stats())
                for (j = 0; j < 7; j++)
                {
                    for (k = 0; k < 16; k++)
                        if ((formes[j] & (1 << k)) != 0)
                            affiche_brique(no_vscr, x_fin[0]+6+x_inc[k]*tc_stats,
                                y_fin[0]-((7-j)*(3*tc_stats))+y_inc[k]*tc_stats, tc_stats, j);
                    /*write_xy_s_c(scr, x_fin[i]+6+4*tc_stats+4,
                        y_fin[i]-((7-j)*(3*tc_stats+2))+2*tc_stats,
                        98, 2, "0");*/
                }
            for (i = 0; i < nbj; i++)
            {
                if ( (cadres) && (view_next > 0) )
                {
                    rectangle(no_vscr, x_next[i], y_next[i],
                        x_next[i]+4*tc_next+5, y_next[i]+4*tc_next+5,
                        couleur_cadre[2]);
                    if (view_next == 2)
                        rectangle(no_vscr, x_next2[i], y_next2[i],
                            x_next2[i]+4*tc_next2+5, y_next2[i]+4*tc_next2+5,
                            couleur_cadre[2]);
                }
                if ( (assombrit) && (view_next > 0) )
                {
                    copy_v2v_xy(no_vscr_fond_sombre, no_vscr,
                        x_next[i]+1, y_next[i]+1,
                        4*tc_next+4, 4*tc_next+4);
                    if (view_next == 2)
                        copy_v2v_xy(no_vscr_fond_sombre, no_vscr,
                            x_next2[i]+1, y_next2[i]+1,
                            4*tc_next2+4, 4*tc_next2+4);
                }
            }
        }
        else
            for (i = 0; i < nbj; i++)
            {
                if (cadres)
                {
                    rectangle(no_vscr, x_deb[i]-1, y_deb[i]-1,
                        x_deb[i]+(x_max[i]-x_min[i]+1)*tc,
                        y_deb[i]+(y_max[i]-y_min[i]+1)*tc, couleur_cadre[ordi[i]]);
                    if (get_window_msg())
                        rectangle(no_vscr, x_deb[i]-1, y_fin[i]+3,
                            x_fin[i], y_fin[i]+3+14, couleur_cadre[ordi[i]]);
                    if (view_next > 0)
                    {
                        rectangle(no_vscr, x_next[i], y_next[i],
                            x_next[i]+4*tc_next+5, y_next[i]+4*tc_next+5,
                            couleur_cadre[ordi[i]]);
                        if (view_next == 2)
                            rectangle(no_vscr, x_next2[i], y_next2[i],
                                x_next2[i]+4*tc_next2+5, y_next2[i]+4*tc_next2+5,
                                couleur_cadre[ordi[i]]);
                    }
                }
                if (assombrit)
                {
                    copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_deb[i], y_deb[i],
                        x_fin[i]-x_deb[i], y_fin[i]-y_deb[i]);
                    if (get_window_msg())
                        copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_deb[i],
                            y_fin[i]+4, x_fin[i]-x_deb[i], 13);
                    if (view_next > 0)
                    {
                        copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_next[i]+1,
                            y_next[i]+1, 4*tc_next+4, 4*tc_next+4);
                        if (view_next == 2)
                            copy_v2v_xy(no_vscr_fond_sombre, no_vscr,
                                x_next2[i]+1, y_next2[i]+1,
                                4*tc_next2+4, 4*tc_next2+4);
                    }
                }
                if (get_stats())
                    for (j = 0; j < 7; j++)
                    {
                        for (k = 0; k < 16; k++)
                            if ((formes[j] & (1 << k)) != 0)
                                affiche_brique(no_vscr, x_fin[i]+6+x_inc[k]*tc_stats,
                                    y_fin[i]-((7-j)*(3*tc_stats))+y_inc[k]*tc_stats, tc_stats, j);
                        /*write_xy_s_c(scr, x_fin[i]+6+4*tc_stats+4,
                            y_fin[i]-((7-j)*(3*tc_stats+2))+2*tc_stats,
                            98, 2, "0");*/
                    }
            }
    }

    /* dessine l'aire de jeu de chaque joueur */
    for (i = 0; i < nbj; i++)
        if ( (i == 0) || (team[i] != team[i-1]) )
        {
            for (j = y_min[team[i]]; j <= y_max[team[i]]; j++)
                for (k = x_min[team[i]]; k <= x_max[team[i]]; k++)
                    if (etat[j][k] != 0)
                        affiche_brique(no_vscr,
                            x_deb[team[i]]+tc*(k-x_min[team[i]]),
                            y_deb[team[i]]+tc*(j-y_min[team[i]]), tc,
                            etat[j][k]-1);
        }

    /* dessine les carres pour nombre de 4, 3, 2, 1 */
    for (i = 0; i < nbj; i++)
        if ( (i == 0) || (team[i] != team[i-1]) )
        {
            for (j = 0; j < 4; j++)
                for (k = 0; k < 4; k++)
                    if (carres_4_lignes[j][k] != -1)
                        affiche_brique(no_vscr, x_fin[team[i]]+3+k*tc_next2, y_carres_lignes[i][3]+j*tc_next2,
                            tc_next2, carres_4_lignes[j][k]);
            for (j = 0; j < 3; j++)
                for (k = 0; k < 4; k++)
                    if (carres_3_lignes[j][k] != -1)
                        affiche_brique(no_vscr, x_fin[team[i]]+3+k*tc_next2, y_carres_lignes[i][2]+j*tc_next2,
                            tc_next2, carres_3_lignes[j][k]);
            for (j = 0; j < 2; j++)
                for (k = 0; k < 4; k++)
                    if (carres_2_lignes[j][k] != -1)
                        affiche_brique(no_vscr, x_fin[team[i]]+3+k*tc_next2, y_carres_lignes[i][1]+j*tc_next2,
                            tc_next2, carres_2_lignes[j][k]);
            for (k = 0; k < 4; k++)
                if (carres_1_ligne[k] != -1)
                    affiche_brique(no_vscr, x_fin[team[i]]+3+k*tc_next2, y_carres_lignes[i][0],
                        tc_next2, carres_1_ligne[k]);
            if (cadres)
            {
                /* cadre pour nombre de lignes faites */
                if (get_game_team())
                    rectangle(no_vscr, x_fin[team[i]]+2, y_carres_lignes[i][0]+7,
                        x_fin[team[i]]+23, y_carres_lignes[i][0]+16, couleur_cadre[2]);
                else
                    rectangle(no_vscr, x_fin[team[i]]+2, y_carres_lignes[i][0]+7,
                        x_fin[team[i]]+23, y_carres_lignes[i][0]+16, couleur_cadre[ordi[i]]);
            }
            if (assombrit)
                copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_fin[team[i]]+3,
                    y_carres_lignes[i][0]+8, 20, 8);
        }

    /* statut en bas de l'ecran */
    if (status_jeu)
    {
        write_xy_s_c(no_vscr, 0, 194, 103, 1, "F10=PAUSE");
        if (sb16)
        {
            write_xy_s_c(no_vscr, 50, 194, 103, 1, "Volume:   %%");
            x_volume_jeu = 50+text_length("Volume:")+1;
            write_xy_s_c(no_vscr, 110, 194, 103, 1, "Musik:");
            x_musique_jeu = 110+text_length("Musik:")+4;
        }
    }

    /* temps */
    if (get_time_limite() > 0)
    {
        if (cadres)
        {
            if (get_game_team())
                rectangle(no_vscr, x_time-2, y_time-2, x_time+19, y_time+7,
                    couleur_cadre[2]);
            else
                rectangle(no_vscr, x_time-2, y_time-2, x_time+19, y_time+7,
                    couleur_cadre[0]);
        }
        if (assombrit)
            copy_v2v_xy(no_vscr_fond_sombre, no_vscr, x_time-1, y_time-1,
                20, 8);
    }
}

/*
 * init_aires_de_jeu: initialise les aires de jeu des joueurs
 *                    Exemple: remplit avec des carres si hauteur_ini > 0
 */

void init_aires_de_jeu()
{
    int     i, j, k, x;

    /* initialise les etats a 0 */
    for (i = 0; i < ETAT_X_MAX; i++)
        for (j = 0; j < ETAT_Y_MAX; j++)
            etat[j][i] = 0;

    /* remplit aleatoirement si hauteur_ini > 0 */
    if (get_hauteur_ini() > 0)
    {
        if (game_mode == GAME_CHALLENGE)
        {
            switch (niveau)
            {
                case 0:
                case 1:
                case 2:
                    for (j = y_max[0]; j >= y_max[0]-hauteur_ini+1; j--)
                    {
                        k = y_max[0]-j+1;
                        for (i = x_min[0]+k; i <= x_max[0]-k; i++)
                            etat[j][i] = (rand()%7)+1;
                    }
                    break;
                case 3:
                case 4:
                case 5:
                    for (j = y_max[0]; j >= y_max[0]-(hauteur_ini*2)+1; j--)
                    {
                        k = (y_max[0]-j)/2+1;
                        for (i = x_min[0]+k; i <= x_max[0]-k; i++)
                            etat[j][i] = (rand()%7)+1;
                    }
                    break;
                case 6:
                case 7:
                case 8:
                    for (j = y_max[0]; j >= y_max[0]-hauteur_ini+1; j--)
                    {
                        k = y_max[0]-j+1;
                        for (i = x_min[0]+k; i <= x_max[0]-k; i++)
                            if ( ( (j == y_max[0]) && (hauteur_ini == 1) )
                                || (j == y_max[0]-hauteur_ini+1)
                                || (i == x_min[0]+k)
                                || (i == x_max[0]-k) )
                                etat[j][i] = (rand()%7)+1;
                    }
                    break;
                case 9:
                case 10:
                case 11:
                    for (j = y_max[0]; j >= y_max[0]-hauteur_ini*2+1; j--)
                    {
                        for (i = x_min[0]; i <= x_max[0]; i++)
                            if (i%2 == 0)
                                etat[j][i] = (rand()%7)+1;
                    }
                    break;
                case 12:
                case 13:
                case 14:
                    for (j = y_max[0]; j >= y_max[0]-hauteur_ini*2+1; j--)
                    {
                        for (i = x_min[0]; i <= x_max[0]; i++)
                            if ( ( (i%2 == 0) && (j%2 == 0) )
                                || ( (i%2 != 0) && (j%2 != 0) ) )
                                etat[j][i] = (rand()%7)+1;
                    }
                    break;
            }
        }
        else
        {
            for (i = 0; i < nbj; i++)
                if ( (i == 0) || (team[i] != team[i-1]) )
                {
                    for (j = y_max[team[i]]; j >= y_max[team[i]]-3*hauteur_ini; j--)
                        for (k = 0; k < ((x_max[team[i]]-x_min[team[i]])*4)/10; k++)
                        {
                            do
                            {
                                x = rand()%(x_max[team[i]]-x_min[team[i]]+1)+x_min[team[i]];
                            } while (etat[j][x] != 0);
                            etat[j][x] = (rand()%7)+1;
                        }
                }
        }
    }

}

/*
 * init_jeu: initialise le jeu
 */

void init_jeu()
{
    int     i;

    if (same_pieces)
    {
        for (i = 0; i < nbj; i++)
            numero_forme[i] = 0;
        for (i = 0; i < NB_FORMES_AV; i++)
            forme_avance[i] = choisit_piece_hasard(); /*rand()%(7+eccentric*9);*/
    }
    else
    {
        for (i = 0; i < nbj; i++)
            next_forme[i] = choisit_piece_hasard(); /*rand()%(7+eccentric*9);*/
    }
}

/*
 * game_write_xy_s_c: affiche une chaine pour le jeu (remet d'abord le fond)
 *                    Si largeur < 0 ==> ecrit a gauche
 *                    Si largeur > 0 ==> aligne a droite
 */

void game_write_xy_s_c(int no_vscr, int x, int y, int largeur,
                       int coul, int degrade, unsigned char *chaine, ...)
{
    unsigned char   buffer[100];
    va_list         argptr;
    int             largeur_recopiee;

    va_start(argptr, chaine);
    vsprintf(buffer, chaine, argptr);
    va_end(argptr);
    largeur_recopiee = abs(largeur);
    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_VSCREEN, x, y, largeur_recopiee, 5);
    else
        copy_v2v_xy(VSCR_VSCREEN, no_vscr, x, y, largeur_recopiee, 5);
    if (largeur > 0)
        write_xy_s_c(no_vscr, x+largeur-text_length(buffer), y, coul, degrade, buffer);
    else
        write_xy_s_c(no_vscr, x, y, coul, degrade, buffer);
}

/*
 * game_write_xy_s_c_o: affiche une chaine pour le jeu (remet d'abord le fond)
 *                    Si largeur < 0 ==> ecrit a gauche
 *                    Si largeur > 0 ==> cale a droite
 */

void game_write_xy_s_c_o(int no_vscr, int x, int y, int largeur,
                         int coul, int degrade, int ombre,
                         unsigned char *chaine, ...)
{
    unsigned char   buffer[100];
    va_list         argptr;
    int             largeur_recopiee;

    va_start(argptr, chaine);
    vsprintf(buffer, chaine, argptr);
    va_end(argptr);
    largeur_recopiee = abs(largeur);
    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_VSCREEN, x, y, largeur_recopiee+1, 6);
    else
        copy_v2v_xy(VSCR_VSCREEN, no_vscr, x, y, largeur_recopiee+1, 6);
    if (largeur > 0)
        write_xy_s_c_o(no_vscr, x+largeur-text_length(buffer), y,
            coul, degrade, ombre, buffer);
    else
        write_xy_s_c_o(no_vscr, x, y, coul, degrade, ombre, buffer);
}

/*
 * game_write_xy_s_c_o_chiffres: affiche une chaine pour le jeu
 *                               (remet d'abord le fond)
 *                               Si largeur < 0 ==> ecrit a gauche
 *                               Si largeur > 0 ==> cale a droite
 * (attention: seulement pour des chiffres, le '1' est affiche sur 3 pixels)
 */

void game_write_xy_s_c_o_chiffres(int no_vscr, int x, int y, int largeur,
                                  int coul, int degrade, int ombre,
                                  unsigned char *chaine, ...)
{
    unsigned char   buffer[100], chaine_tmp[2];
    va_list         argptr;
    int             largeur_recopiee, i, j;

    va_start(argptr, chaine);
    vsprintf(buffer, chaine, argptr);
    va_end(argptr);
    largeur_recopiee = abs(largeur);
    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_VSCREEN, x, y, largeur_recopiee+1, 6);
    else
        copy_v2v_xy(VSCR_VSCREEN, no_vscr, x, y, largeur_recopiee+1, 6);
    chaine_tmp[1] = '\0';
    for (i = 0; i < strlen(buffer); i++)
    {
        if (largeur > 0)
        {
            chaine_tmp[0] = buffer[strlen(buffer)-i-1];
            j = (chaine_tmp[0] == '1')?1:0;
            write_xy_s_c_o(no_vscr, x+largeur-4*(i+1)+j+1, y,
                coul, degrade, ombre, chaine_tmp);
        }
        else
        {
            chaine_tmp[0] = buffer[i];
            j = (chaine_tmp[0] == '1')?1:0;
            write_xy_s_c_o(no_vscr, x+4*i+j, y, coul, degrade, ombre, chaine_tmp);
        }
    }
}

/*
 * ajoute_msg: ajoute un message dans la fenetre sous l'aire d'un joueur
 */

void ajoute_msg(int joueur, int s_efface_tout_seul, unsigned char *chaine, ...)
{
    unsigned char   buffer[100];
    va_list         argptr;

    if (get_window_msg())
    {
        va_start(argptr, chaine);
        vsprintf(buffer, chaine, argptr);
        va_end(argptr);

        if (msg[joueur][0][0] == '\0')
            /* 1er message */
            strcpy(msg[joueur][0], buffer);
        else
        {
            strcpy(msg[joueur][1], msg[joueur][0]);
            strcpy(msg[joueur][0], buffer);
        }
        copy_v2s_xy(VSCR_AUTRES+2, x_deb[team[joueur]],
            y_fin[team[joueur]]+4, x_fin[team[joueur]]-x_deb[team[joueur]], 13);

        write_xy_s_c(ECRAN_REEL, x_deb[team[joueur]]+1, y_fin[team[joueur]]+3+2,
            92, 2, msg[joueur][0]);
        write_xy_s_c(ECRAN_REEL, x_deb[team[joueur]]+1, y_fin[team[joueur]]+3+2+6,
            74, 1, msg[joueur][1]);

        if (s_efface_tout_seul)
        {
            if (msg[joueur][1][0] != '\0')
                cptr_msg[joueur] = 800;
            else
                cptr_msg[joueur] = 400;
        }
        else
            cptr_msg[joueur] = 0;
    }
}

/*
 * efface_msgs: efface tous les messages d'un joueur
 */

void efface_msgs(int joueur)
{
    if (get_window_msg())
    {
        msg[joueur][0][0] = '\0';
        msg[joueur][1][0] = '\0';
        copy_v2s_xy(VSCR_AUTRES+2, x_deb[team[joueur]],
            y_fin[team[joueur]]+4, x_fin[team[joueur]]-x_deb[team[joueur]], 13);
    }
}

/*
 * affiche_nombre_lignes: affiche le nombre de lignes total effectuees par un
 *                        joueur (ou une equipe)
 */

void affiche_nombre_lignes(int joueur)
{
    game_write_xy_s_c_o(ECRAN_REEL, x_fin[team[joueur]]+4,
        y_carres_lignes[team[joueur]][0]+9, 17, 88, 2, 26,
        "%d", nb_lignes[team[joueur]][0]);
}

/*
 * efface_nombre_lignes: efface le nombre de lignes total effectuees par un
 *                       joueur (ou une equipe)
 */

void efface_nombre_lignes(int joueur)
{
    copy_v2s_xy(VSCR_VSCREEN, x_fin[team[joueur]]+4,
        y_carres_lignes[team[joueur]][0]+9, 18, 6);
}

/*
 * affiche_temps: affiche le temps restant ou ecoule
 */

void affiche_temps()
{
    unsigned char   chaine[20];

    if (cptr_time >= 3600)  /* plus d'une heure: pas mal ! */
    {
        sprintf(chaine, "% 2d", cptr_time/3600);
        game_write_xy_s_c_o_chiffres(ECRAN_REEL, x_time, y_time,
            7, 61, -1, 27, chaine);
        if (cptr_time%2 != 0)
            copy_v2s_xy(VSCR_VSCREEN, x_time+8, y_time, 2, 6);
        else
            write_xy_s_c_o(ECRAN_REEL, x_time+8, y_time, 61, -1, 27, ":");
        sprintf(chaine, "%02d", (cptr_time%3600)/60);
        game_write_xy_s_c_o_chiffres(ECRAN_REEL, x_time+10, y_time,
            7, 61, -1, 27, chaine);
    }
    else
    {
        sprintf(chaine, "% 2d", cptr_time/60);
        game_write_xy_s_c_o_chiffres(ECRAN_REEL, x_time, y_time,
            7, 61, -1, 27, chaine);
        write_xy_s_c_o(ECRAN_REEL, x_time+8, y_time, 61, -1, 27, ":");
        sprintf(chaine, "%02d", cptr_time%60);
        game_write_xy_s_c_o_chiffres(ECRAN_REEL, x_time+10, y_time,
            7, 61, -1, 27, chaine);
    }
}

/*
 * affiche_piece_xy: affiche une piece aux coordonnees EXACTES indiquees dans
 *                   la structure piece (et non pas en partant du bord gauche
 *                   du joueur)
 */

void affiche_piece_xy(int no_vscr, int joueur, t_piece *p)
{
    int     i;

    switch (p->p_bonus)
    {
        case PIECE_BOMBE:
            putsprite(no_vscr, &bombe[p->p_taille-TC_MIN][image_bombe/10],
                p->p_x+p->p_taille, p->p_y+p->p_taille);
            break;
        default:
            for (i = 0; i < 16; i++)
                if ((p->p_forme & (1 << i)) != 0)
                {
                    affiche_brique(no_vscr, p->p_x+x_inc[i]*p->p_taille,
                        p->p_y+y_inc[i]*p->p_taille, p->p_taille, p->p_couleur);
                }
    }
}

/*
 * game_affiche_brique: affiche une brique dans une aire de jeu
 *                      en tenant compte du fait que l'ecran peut etre inverse
 */

void game_affiche_brique(int no_vscr, int joueur, int xb, int yb,
                         int taille_carre, int couleur_carre)
{
    if (ecran_inverse[joueur])
        affiche_brique(no_vscr, x_fin[team[joueur]]-(xb+1)*taille_carre,
            y_fin[team[joueur]]-(yb+1)*taille_carre, taille_carre, couleur_carre);
    else
        affiche_brique(no_vscr, x_deb[team[joueur]]+xb*taille_carre,
            y_deb[team[joueur]]+yb*taille_carre, taille_carre, couleur_carre);
}

/*
 * game_efface_brique: efface une brique dans une aire de jeu
 *                     en tenant compte du fait que l'ecran peut etre inverse
 */

void game_efface_brique(int no_vscr, int joueur, int xb, int yb,
                        int taille_carre)
{
    if (ecran_inverse[joueur])
        efface_brique(no_vscr, VSCR_VSCREEN, x_fin[team[joueur]]-(xb+1)*taille_carre,
            y_fin[team[joueur]]-(yb+1)*taille_carre, taille_carre);
    else
        efface_brique(no_vscr, VSCR_VSCREEN, x_deb[team[joueur]]+xb*taille_carre,
            y_deb[team[joueur]]+yb*taille_carre, taille_carre);
}

/*
 * game_affiche_piece_bonus: affiche une piece bonus en tenant compte que
 *                           l'ecran peut etre inverse
 */

void game_affiche_piece_bonus(int no_vscr, int joueur, t_piece *p)
{
    /*  mettre la bombe inversee */
    if (ecran_inverse[joueur])
        putsprite(no_vscr, &bombe[p->p_taille-TC_MIN][image_bombe/10],
            x_fin[team[joueur]]-(p->p_x-x_min[team[joueur]]+1)*p->p_taille-p->p_taille,
            y_fin[team[joueur]]-(p->p_y-y_min[team[joueur]]+1)*p->p_taille-p->p_taille);
    else
        putsprite(no_vscr, &bombe[p->p_taille-TC_MIN][image_bombe/10],
            x_deb[team[joueur]]+(p->p_x-x_min[team[joueur]])*p->p_taille+p->p_taille,
            y_deb[team[joueur]]+(p->p_y-y_min[team[joueur]])*p->p_taille+p->p_taille);
}

/*
 * affiche_piece: affiche une piece d'un joueur
 */

void affiche_piece(int no_vscr, int joueur, t_piece *p)
{
    int     i;

    switch (p->p_bonus)
    {
        case PIECE_BOMBE:
            game_affiche_piece_bonus(no_vscr, joueur, p);
            break;
        default:
            for (i = 0; i < 16; i++)
                if ((p->p_forme & (1 << i)) != 0)
                {
                    game_affiche_brique(no_vscr, joueur,
                        p->p_x-x_min[team[joueur]]+x_inc[i],
                        p->p_y-y_min[team[joueur]]+y_inc[i],
                        p->p_taille, p->p_couleur);
                }
    }
}

/*
 * efface_piece: efface une piece d'un joueur
 */

void efface_piece(int no_vscr, int joueur, t_piece *p)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ((p->p_forme & (1 << i)) != 0)
            game_efface_brique(no_vscr, joueur,
                p->p_x-x_min[team[joueur]]+x_inc[i],
                p->p_y-y_min[team[joueur]]+y_inc[i], p->p_taille);
}

/*
 * redessine_aire_jeu: redessine l'aire de jeu complete d'un joueur ou d'une equipe
 */

void redessine_aire_jeu(int joueur)
{
    int     i, j;

    for (j = y_min[team[joueur]]; j <= y_max[team[joueur]]; j++)
        for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
            if ( (abs(etat[j][i]) == ETAT_PIECE_BONUS) || (etat[j][i] == 0) )
                game_efface_brique(ECRAN_REEL, joueur, i-x_min[team[joueur]],
                    j-y_min[team[joueur]], piece[joueur].p_taille);
            else
                game_affiche_brique(ECRAN_REEL, joueur,
                    i-x_min[team[joueur]], j-y_min[team[joueur]],
                    piece[joueur].p_taille, abs(etat[j][i])-1);
}

/* ==========================================================================

                                SPECIFIQUE X-WINDOW

   ========================================================================== */

#ifndef MSDOS
void X_redraw_all()
{
    int     i, j, k;

    /*X_dessine_flashtris(10, 10, 8, 14, 12, 7);*/
    for (i = 0; i < nbj; i++)
    {
        rectangle(ECRAN_REEL, x_deb[team[i]]-1, y_deb[team[i]]-1,
            x_deb[team[i]]+(x_max[team[i]]-x_min[team[i]]+1)*piece[i].p_taille,
            y_deb[team[i]]+(y_max[team[i]]-y_min[team[i]]+1)*piece[i].p_taille, 12);
        for (j = x_min[team[i]]; j <= x_max[team[i]]; j++)
            for (k = y_min[team[i]]; k <= y_max[team[i]]; k++)
                if (etat[k][j] == 0)
                    efface_brique(ECRAN_REEL, VSCR_VSCREEN,
                        x_deb[team[i]]+(j-x_min[team[i]])*piece[i].p_taille,
                        y_deb[team[i]]+(k-y_min[team[i]])*piece[i].p_taille,
                        piece[i].p_taille);
                else
                    affiche_brique(ECRAN_REEL,
                        x_deb[team[i]]+(j-x_min[team[i]])*piece[i].p_taille,
                        y_deb[team[i]]+(k-y_min[team[i]])*piece[i].p_taille,
                        piece[i].p_taille, etat[k][j]-1);
        /*write_xy_c_f(0, i*font_height, 4, 0, "%02d", nb_lignes[i]);*/
        affiche_piece(ECRAN_REEL, i, &piece[i]);
    }
}
#endif

#ifndef MSDOS
/*void X_traite_evenement()
{
    XEvent      event;
    int         i;

    XNextEvent(display, &event);
    switch (event.type)
    {
        case Expose:
            X_redraw_all();
            break;
        case ConfigureNotify:
            if ( (event.xconfigure.width != scr_nbx)
                || (event.xconfigure.height != scr_nby) )
            {
                scr_nbx = event.xconfigure.width;
                scr_nby = event.xconfigure.height;
                tc = 1;
                while ((10*tc+5)*nbj < scr_nbx) tc++;
                if (tc > 1) tc--;
                for (i = 0; i < nbj; i++)
                {
                    x_deb[team[i]] = 1+(10*tc+5)*i;
                    y_deb[team[i]] = 1;
                    piece[i].p_taille = tc;
                }
                XClearWindow(event.xconfigure.display, event.xconfigure.window);
                X_redraw_all();
            }
            break;
        case ClientMessage:
            quitte_jeu = 1;
            break;
        case KeyPress:
            switch (event.xkey.keycode)
            {
                case 9:     // Echap
                    key_tab[1] = 1;
                    break;
                case 36:    // Enter
                    key_tab[28] = 1;
                    break;
                case 82:    // moins (pave numerique)
                    if (volume > 0)
                        volume--;
                    setvolume(volume);
                    break;
                case 86:    // plus (pave numerique)
                    if (volume < 99)
                        volume++;
                    setvolume(volume);
                    break;
                case 83:    // 4
                    key_tab[75] = 1;
                    break;
                case 84:    // 5
                    key_tab[76] = 1;
                    break;
                case 85:    // 6
                    key_tab[77] = 1;
                    break;
                case 88:
                    key_tab[80] = 1;
                    break;
                default:
                    printf("keycode = %d\n", event.xkey.keycode);
                    break;
            }
            break;
    }
}*/
#endif

/* ==========================================================================

                                      LE JEU

   ========================================================================== */

/*
 * high_scores: affiche les high scores
 */

void high_scores()
{
    unsigned char   pal_fondu[768];
    unsigned char   chaine[50];
    int             i, j, y;
    /*int             tab_coul[5] = { 142, 199, 66, 15, 112 };*/
    int             tab_coul1[5] = { 230, 230, 163, 163, 94 };
    int             tab_coul2[5] = { 142, 142, 199, 199, 66 };
    unsigned char   *libelle_high_score[2][9] =
                    { { "RANK", "NAME", "LINES",  "#4",  "#3",  "#2",  "#1",  "SCORE", "TIME" },
                      { "POS", "NOM", "LIGNES", "NB4", "NB3", "NB2", "NB1", "SCORE", "TEMPS" } };

    /*memset(VSCR_AUTRES+3, 0, scr_nbx*scr_nby);
    for (i = 0; i < 10; i++)
    {
        y = 50+i*10;
        sprintf(chaine, "%d.", i+1);
        write_xy_s_c_o_droite(VSCR_AUTRES+3, 5, y, 60, 1, 70, 10, chaine);
        strcpy(chaine, high_score[0][i].hs_nom);
        write_xy_s_c_o(VSCR_AUTRES+3, 20, y, 60, 1, 70, chaine);
        sprintf(chaine, "%d", high_score[0][i].hs_nblignes);
        write_xy_s_c_o_droite(VSCR_AUTRES+3, 115, y, 60, 1, 70, 20, chaine);
        for (j = 0; j < 4; j++)
        {
            sprintf(chaine, "%d", high_score[0][i].hs_nblig[3-j]);
            write_xy_s_c_o_droite(VSCR_AUTRES+3, 145+j*30, y, 60, 1, 70, 20, chaine);
        }
        sprintf(chaine, "%d", high_score[0][i].hs_score);
        write_xy_s_c_o_droite(VSCR_AUTRES+3, 235, y, 60, 1, 70, 25, chaine);
        sprintf(chaine, "%d", high_score[0][i].hs_secondes);
        write_xy_s_c_o_droite(VSCR_AUTRES+3, 270, y, 60, 1, 70, 25, chaine);
    }
    copy_v2s(VSCR_AUTRES+3);
    sleep(1);*/

    memset(pal_fondu, 0, 768);
    clear_screen();
    setpal(pal_fondu);
    load_dat("score.dat", VSCR_VSCREEN);

    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 0, 52, tab_coul1, 0, 24, libelle_high_score[langue][0]);
    write_xy_s_c_o_tab(VSCR_VSCREEN, 32, 52, tab_coul1, 0, libelle_high_score[langue][1]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 101, 52, tab_coul1, 0, 30, libelle_high_score[langue][2]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 141, 52, tab_coul1, 0, 20, libelle_high_score[langue][3]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 161, 52, tab_coul1, 0, 20, libelle_high_score[langue][4]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 181, 52, tab_coul1, 0, 20, libelle_high_score[langue][5]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 201, 52, tab_coul1, 0, 20, libelle_high_score[langue][6]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 236, 52, tab_coul1, 0, 25, libelle_high_score[langue][7]);
    write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 276, 52, tab_coul1, 0, 25, libelle_high_score[langue][8]);

    for (i = 0; i < 10; i++)
    {
        y = 64+i*12;
        sprintf(chaine, "%d.", i+1);
        write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 14, y, tab_coul2, 0, 10, chaine);
        strcpy(chaine, high_score[0][i].hs_nom);
        write_xy_s_c_o_tab(VSCR_VSCREEN, 32, y, tab_coul2, 0, chaine);
        sprintf(chaine, "%d", high_score[0][i].hs_nblignes);
        write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 111, y, tab_coul2, 0, 20, chaine);
        for (j = 0; j < 4; j++)
        {
            sprintf(chaine, "%d", high_score[0][i].hs_nblig[3-j]);
            write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 141+j*20, y, tab_coul2, 0, 20, chaine);
        }
        sprintf(chaine, "%d", high_score[0][i].hs_score);
        write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 236, y, tab_coul2, 0, 25, chaine);
        /*sprintf(chaine, "%d", high_score[0][i].hs_secondes);*/
        /*write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 276, y, tab_coul2, 0, 25, chaine);*/
        sprintf(chaine, "% 2d:%02d:%02d", (high_score[0][i].hs_secondes/3600),
            (high_score[0][i].hs_secondes%3600)/60,
            (high_score[0][i].hs_secondes%3600)%60);
        write_xy_s_c_o_tab_droite(VSCR_VSCREEN, 261, y, tab_coul2, 0, 40, chaine);
    }

    copy_v2s(VSCR_VSCREEN);
    for (j = 0; j < 64; j++)
    {
        for (i = 0; i < 768; i++)
            if (pal_fondu[i] < palette[i])
                pal_fondu[i]++;
            if ((j % 2) == 0)
            {
                waitretrace();
                setpal(pal_fondu);
            }
    }
    setpal(palette);

    /*for (i = 0; i < 10; i++)
        write_xy_s_c(ECRAN_REEL, 5, 50+i*10, 50, 1, ligne_score[i]);*/

    #ifndef MSDOS
    if (key_tab[34])
        save_graphics_raw();
    sleep(10);
    #else
    compteur_usleep_dos = 0;
    while (compteur_usleep_dos < 91*10) ;
    #endif
    clear_screen();
    memcpy(palette, pal_ftris, 768);
    setpal(pal_ftris);
}

/*
 * challenge_init_niveau: initialise le niveau et affiche l'objectif
 */

void challenge_init_niveau()
{
    unsigned char   chaine[4][50];
    int             i, largeur;

    challenge_ready_affiche = 1;
    challenge_timer_ready = 4*91;
    nb_lignes[0][0] = challenge[niveau].c_nb_lignes;
    affiche_nombre_lignes(0);
    cptr_time = challenge[niveau].c_temps_limite;
    affiche_temps();
    annule_etats(0, &piece[0]);
    if (niveau > 0)
        init_aires_de_jeu();
    redessine_aire_jeu(0);
    etats_negatifs(0, &piece[0]);
    affiche_piece(ECRAN_REEL, 0, &piece[0]);
    copy_s2v_xy(VSCR_AUTRES+3, x_deb[0], y_deb[0], x_fin[0]-x_deb[0],
        y_fin[0]-y_deb[0]);
    strcpy(chaine[0], "Challenge");
    if (langue == ANGLAIS)
    {
        sprintf(chaine[1], "Level %d:", niveau+1);
        if (challenge[niveau].c_temps_limite == 0)
            sprintf(chaine[2], "%d lines", challenge[niveau].c_nb_lignes);
        else
        {
            if (challenge[niveau].c_temps_limite%60 == 0)
                sprintf(chaine[2], "%d lines, %d min",
                    challenge[niveau].c_nb_lignes,
                    challenge[niveau].c_temps_limite/60);
            else
                sprintf(chaine[2], "%d lines, %d min %d",
                    challenge[niveau].c_nb_lignes,
                    challenge[niveau].c_temps_limite/60,
                    challenge[niveau].c_temps_limite%60);
        }
        strcpy(chaine[3], "Ready ?");
    }
    else
    {
        sprintf(chaine[1], "Niveau %d:", niveau+1);
        if (challenge[niveau].c_temps_limite == 0)
            sprintf(chaine[2], "%d lignes", challenge[niveau].c_nb_lignes);
        else
        {
            if (challenge[niveau].c_temps_limite%60 == 0)
                sprintf(chaine[2], "%d lignes, %d min",
                    challenge[niveau].c_nb_lignes,
                    challenge[niveau].c_temps_limite/60);
            else
                sprintf(chaine[2], "%d lignes, %d min %d",
                    challenge[niveau].c_nb_lignes,
                    challenge[niveau].c_temps_limite/60,
                    challenge[niveau].c_temps_limite%60);
        }
        strcpy(chaine[3], "Pret ?");
    }

    for (i = 0; i < 4; i++)
    {
        largeur = text_length(chaine[i]);
        write_xy_s_c_o(ECRAN_REEL,
            x_deb[0]+((x_fin[0]-x_deb[0]-largeur)/2),
            y_deb[0]+((y_fin[0]-y_deb[0])/2)+y_texte_ready[i],
            coul_texte_ready[i][0], coul_texte_ready[i][1],
            coul_texte_ready[i][2], chaine[i]);
    }
    ajoute_msg(0, 0, chaine[2]);
    ajoute_msg(0, 0, msg[0][1]);
}

/*
 * challenge_copie_ecran_apres_ready: copie l'ecran du jeu apres affichage
 *                                    du "Ready ?"
 */

void challenge_copie_ecran_apres_ready()
{
    copy_v2s_xy(VSCR_AUTRES+3, x_deb[0], y_deb[0],
        x_fin[0]-x_deb[0], y_fin[0]-y_deb[0]);
}

/*
 * change_niveau: change de niveau
 */

void change_niveau(int new_niveau)
{
    int     msg_s_efface;

    niveau = new_niveau;
    if (game_mode == GAME_CHALLENGE)
    {
        efface_msgs(0);
        msg_s_efface = 0;
    }
    else
        msg_s_efface = 1;
    if (niveau == NIVEAU_MAX)
    {
        if (langue == ANGLAIS)
            ajoute_msg(0, msg_s_efface, "Last level: %d", niveau+1);
        else
            ajoute_msg(0, msg_s_efface, "Dernier niveau: %d", niveau+1);
    }
    else
    {
        if (langue == ANGLAIS)
            ajoute_msg(0, msg_s_efface, "Level %d", niveau+1);
        else
            ajoute_msg(0, msg_s_efface, "Niveau %d", niveau+1);
    }
}

/*
 * choisit_piece_hasard: choisit une piece au hasard (peut etre une piece bonus)
 */

int choisit_piece_hasard()
{
    /*if ( (get_bonus_level()) && (rand()%5 == 2) )
        return 17;
    else*/
        return rand()%(7+eccentric*9);
}

/*
 * verifie_gagne_perdu: verifie qui a gagne/perdu en tenant compte du reseau
 */

void verifie_gagne_perdu()
{
    int     i, nbj_reste;

    if ( (tous_perdu) || (partie_finie) )
        return;

    tous_perdu = 1;
    nbj_reste = 0;
    for (i = 0; i < nbj; i++)
        if (!perdu[team[i]])
        {
            tous_perdu = 0;
            nbj_reste++;
        }

    if ( ( (network) && (tous_perdu) )
        || ( (!network) && ( (tous_perdu)
                        || ( (nbj_reste == 1) && (nbj > 1) ) ) ) )
        partie_finie = 1;

    if ( (network) && (!partie_finie) )
    {
        partie_finie = 1;
        for (i = 0; i < nb_PC_net; i++)
            if ( (i != numero_PC_net) && (PC_net[i].adresse != 0)
                && (PC_net[i].nb_perdus != PC_net[i].nbj) )
                partie_finie = 0;
        /* si partie_finie vaut 1 ici, cela signifie que tous les PC */
        /* du reseau ont perdu sauf le mien !                        */
    }
    if ( (network) && (partie_finie) )
        envoi_perdu();

    if (partie_finie)
    {
        coul_winner = COUL_1_WINNER;
        dir_coul_winner = 1;
    }
}

/*
 * joueur_ou_team_perd: un joueur ou une equipe perd...
 */

void joueur_ou_team_perd(int joueur)
{
    int     i, j;

    for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
        for (j = y_min[team[joueur]]; j <= y_max[team[joueur]]; j++)
            affiche_brique(ECRAN_REEL,
                x_deb[team[joueur]]+(i-x_min[team[joueur]])*piece[joueur].p_taille,
                y_deb[team[joueur]]+(j-y_min[team[joueur]])*piece[joueur].p_taille,
                piece[joueur].p_taille, rand()%7);
    perdu[team[joueur]] = 1;
    efface_msgs(team[joueur]);
    if (game_mode == GAME_CHALLENGE)
    {
        if (langue == ANGLAIS)
            ajoute_msg(team[joueur], 0, "Level %d", niveau+1);
        else
            ajoute_msg(team[joueur], 0, "Niveau %d", niveau+1);
    }
    ajoute_msg(team[joueur], 0, "- GAME OVER -");
    verifie_gagne_perdu();
}

/*
 * retourne_piece: retourne la piece d'un quart de tour dans le sens inverse
 *                 des aiguilles d'une montre
 *                 La piece n'est pas retournee si ce n'est pas possible
 *                 (sortie de l'ecran de jeu ou mur/joueur qui bloque ou piece bonus)
 *                 Si numero de joueur = 255 alors aucun test n'est effectue
 */

void retourne_piece(int joueur, t_piece *p)
{
    int     ancienne_forme;
    int     i;

    /* piece bonus ? */
    if (p->p_bonus != 0)
        return;

    ancienne_forme = p->p_forme;

    switch (p->p_forme)
    {
        case 2048+1024+512+256:p->p_forme = 8192+512+32+2; break;
        case 8192+512+32+2:p->p_forme = 2048+1024+512+256; break;

        case 2048+1024+512+64:p->p_forme = 16384+1024+512+64; break;
        case 16384+1024+512+64:p->p_forme = 16384+2048+1024+512; break;
        case 16384+2048+1024+512:p->p_forme = 16384+2048+1024+64; break;
        case 16384+2048+1024+64:p->p_forme = 2048+1024+512+64; break;

        case 2048+1024+512+128:p->p_forme = 16384+1024+64+32; break;
        case 16384+1024+64+32:p->p_forme = 512+128+64+32; break;
        case 512+128+64+32:p->p_forme = 32768+16384+1024+64; break;
        case 32768+16384+1024+64:p->p_forme = 2048+1024+512+128; break;

        case 2048+1024+512+32:p->p_forme = 16384+8192+1024+64; break;
        case 16384+8192+1024+64:p->p_forme = 2048+128+64+32; break;
        case 2048+128+64+32:p->p_forme = 16384+1024+128+64; break;
        case 16384+1024+128+64:p->p_forme = 2048+1024+512+32; break;

        case 1024+512+128+64:p->p_forme = 16384+1024+512+32; break;
        case 16384+1024+512+32:p->p_forme = 1024+512+128+64; break;

        case 2048+1024+64+32:p->p_forme = 8192+1024+512+64; break;
        case 8192+1024+512+64:p->p_forme = 2048+1024+64+32; break;

        case 1024+512+32:p->p_forme = 1024+512+64; break;
        case 1024+512+64:p->p_forme = 1024+64+32; break;
        case 1024+64+32:p->p_forme = 512+64+32; break;
        case 512+64+32:p->p_forme = 1024+512+32; break;

        case 2048+1024+512+256+128+64+32+16:p->p_forme = 16384+8192+1024+512+64+32+4+2; break;
        case 16384+8192+1024+512+64+32+4+2:p->p_forme = 2048+1024+512+256+128+64+32+16; break;

        case 2048+1024+512+64+4:p->p_forme = 16384+1024+512+256+64; break;
        case 16384+1024+512+256+64:p->p_forme = 16384+1024+128+64+32; break;
        case 16384+1024+128+64+32:p->p_forme = 8192+2048+1024+512+32; break;
        case 8192+2048+1024+512+32:p->p_forme = 2048+1024+512+64+4; break;

        case 1024+512+128:p->p_forme = 16384+1024+32; break;
        case 16384+1024+32:p->p_forme = 512+128+64; break;
        case 512+128+64:p->p_forme = 32768+1024+64; break;
        case 32768+1024+64:p->p_forme = 1024+512+128; break;

        case 2048+1024+32:p->p_forme = 8192+1024+64; break;
        case 8192+1024+64:p->p_forme = 2048+64+32; break;
        case 2048+64+32:p->p_forme = 16384+1024+128; break;
        case 16384+1024+128:p->p_forme = 2048+1024+32; break;

        case 512+256+128+64:p->p_forme = 16384+1024+32+2; break;
        case 16384+1024+32+2:p->p_forme = 512+256+128+64; break;

        case 2048+1024+32+16:p->p_forme = 8192+512+64+4; break;
        case 8192+512+64+4:p->p_forme = 2048+1024+32+16; break;

        /*case 2048+1024+512:p->p_forme = 512+32+2; break;*/
        /*case 512+32+2:p->p_forme = 2048+1024+512; break;*/
        case 2048+1024+512:p->p_forme = 16384+1024+64; break;
        case 16384+1024+64:p->p_forme = 2048+1024+512; break;

        case 2048+1024+512+256+64+32:p->p_forme = 16384+1024+512+64+32+4; break;
        case 16384+1024+512+64+32+4:p->p_forme = 16384+8192+2048+1024+512+256; break;
        case 16384+8192+2048+1024+512+256:p->p_forme = 16384+2048+1024+128+64+4; break;
        case 16384+2048+1024+128+64+4:p->p_forme = 2048+1024+512+256+64+32; break;
        default:
            p->p_forme = (((p->p_forme) & 8) >> 3)+         /* rangee du bas */
                         (((p->p_forme) & 128) >> 6)+
                         (((p->p_forme) & 2048) >> 9)+
                         (((p->p_forme) & 32768) >> 12)+
                         (((p->p_forme) & 4) << 2)+         /* rangee milieu bas */
                         (((p->p_forme) & 64) >> 1)+
                         (((p->p_forme) & 1024) >> 4)+
                         (((p->p_forme) & 16384) >> 7)+
                         (((p->p_forme) & 2) << 7)+         /* rangee milieu haut */
                         (((p->p_forme) & 32) << 4)+
                         (((p->p_forme) & 512) << 1)+
                         (((p->p_forme) & 8192) >> 2)+
                         (((p->p_forme) & 1) << 12)+        /* rangee du haut */
                         (((p->p_forme) & 16) << 9)+
                         (((p->p_forme) & 256) << 6)+
                         (((p->p_forme) & 4096) << 3);
    }

    if (joueur != 255)
        for (i = 0; i < 16; i++)
            if ( ((p->p_forme & (1 << i)) != 0) &&
                ( (p->p_x+x_inc[i] < x_min[team[joueur]])
                || (p->p_x+x_inc[i] > x_max[team[joueur]])
                || (p->p_y+y_inc[i] > y_max[team[joueur]])
                || (etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] != 0) ) )
            {
                p->p_forme = ancienne_forme;
                return;
            }
}

/*
 * etats_negatifs: force les etats de la piece a la couleur de la piece en negatif
 */

void etats_negatifs(int joueur, t_piece *p)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ( ((p->p_forme & (1 << i)) != 0)
            && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
            etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = -(p->p_couleur+1);
}

/*
 * etats_negatifs_team: force les etats des pieces de l'equipe du joueur
 *                      a la couleur de la piece en negatif
 */

void etats_negatifs_team(int joueur)
{
    int         i, j;
    t_piece     *p;

    for (j = 0; j < nbj; j++)
    {
        if ( (team[j] == team[joueur]) && (!perdu[team[j]]) )
        {
            p = &piece[j];
            for (i = 0; i < 16; i++)
                if ( ((p->p_forme & (1 << i)) != 0)
                    && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
                    etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = -(p->p_couleur+1);
        }
    }
}

/*
 * etats_positifs: force les etats de la piece a la couleur de la piece en positif
 */

void etats_positifs(int joueur, t_piece *p)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ( ((p->p_forme & (1 << i)) != 0)
            && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
            etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = p->p_couleur+1;
}

/*
 * etats_positifs_team: force les etats des pieces de l'equipe du joueur
 *                      a la couleur de la piece en positif
 */

void etats_positifs_team(int joueur)
{
    int         i, j;
    t_piece     *p;

    for (j = 0; j < nbj; j++)
    {
        if ( (team[j] == team[joueur]) && (!perdu[team[j]]) )
        {
            p = &piece[j];
            for (i = 0; i < 16; i++)
                if ( ((p->p_forme & (1 << i)) != 0)
                    && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
                    etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = p->p_couleur+1;
        }
    }
}

/*
 * annule_etats: annule les etats negatifs pour la piece (cf etats_negatifs)
 */

void annule_etats(int joueur, t_piece *p)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ( ((p->p_forme & (1 << i)) != 0)
            && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
            etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = 0;
}

/*
 * annule_etats_team: annule les etats negatifs pour les pieces de l'equipe du joueur
 */

void annule_etats_team(int joueur)
{
    int         i, j;
    t_piece     *p;

    for (j = 0; j < nbj; j++)
    {
        if ( (team[j] == team[joueur]) && (!perdu[team[j]]) )
        {
            p = &piece[j];
            for (i = 0; i < 16; i++)
                if ( ((p->p_forme & (1 << i)) != 0)
                    && (p->p_y+y_inc[i] >= y_min[team[joueur]]) )
                    etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] = 0;
        }
    }
}

/*
 * peut_descendre: renvoie 0 si la piece peut descendre
 *                         1 si elle est bloquee par un mur (ou le bas de l'ecran)
 *                         2 si une autre piece bloque la descente (mode cooperatif)
 */

int peut_descendre(int joueur, t_piece *p)
{
    int     puiss2[4][4] =
            { { 32768, 16384, 8192, 4096 }, { 2048, 1024, 512, 256 },
              { 128, 64, 32, 16 }, { 8, 4, 2, 1 } };
    int     i, plus_bas, colonne_possible[4];

    for (i = 0; i < 4; i++)
    {
        plus_bas = 3;
        while ( (plus_bas >= 0) && ((p->p_forme & puiss2[plus_bas][i]) == 0) )
            plus_bas--;
        if (plus_bas >= 0)
        {
            if ( (p->p_y+plus_bas >= y_max[team[joueur]])
                || (etat[p->p_y+plus_bas+1][p->p_x+i] > 0) )
                return 1;
            colonne_possible[i] = (etat[p->p_y+plus_bas+1][p->p_x+i] == 0);
        }
        else
            colonne_possible[i] = 1;
    }
    for (i = 0; i < 4; i++)
        if (!colonne_possible[i])
            return 2;
    return 0;
}

/*
 * peut_afficher: renvoie 1 si la piece peut etre affichee, 0 sinon (=perdu)
 */

int peut_afficher(int joueur, t_piece *p)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ( ((p->p_forme & (1 << i)) != 0)
            && (etat[p->p_y+y_inc[i]][p->p_x+x_inc[i]] != 0) )
            return 0;
    return 1;
}

/*
 * peut_aller_gauche: renvoie 1 si il est possible d'aller a gauche, 0 sinon
 */

int peut_aller_gauche(int joueur, t_piece *p)
{
    int     puiss2[4][4] =
            { { 32768, 16384, 8192, 4096 }, { 2048, 1024, 512, 256 },
              { 128, 64, 32, 16 }, { 8, 4, 2, 1 } };
    int     i, plus_gauche;

    for (i = 0; i < 4; i++)
    {
        plus_gauche = 0;
        while ( (plus_gauche < 4)
            && ((p->p_forme & puiss2[i][plus_gauche]) == 0) )
            plus_gauche++;
        if ( (plus_gauche < 4) &&
            ( (p->p_x+plus_gauche-1 < x_min[team[joueur]])
            || (etat[p->p_y+i][p->p_x+plus_gauche-1] != 0) ) )
            return 0;
    }
    return 1;
}

/*
 * peut_aller_droite: renvoie 1 si il est possible d'aller a droite, 0 sinon
 */

int peut_aller_droite(int joueur, t_piece *p)
{
    int     puiss2[4][4] =
            { { 32768, 16384, 8192, 4096 }, { 2048, 1024, 512, 256 },
              { 128, 64, 32, 16 }, { 8, 4, 2, 1 } };
    int     i, plus_droite;

    for (i = 0; i < 4; i++)
    {
        plus_droite = 3;
        while ( (plus_droite >= 0)
            && ((p->p_forme & puiss2[i][plus_droite]) == 0) )
            plus_droite--;
        if ( (plus_droite >= 0) &&
            ( (p->p_x+plus_droite+1 > x_max[team[joueur]])
            || (etat[p->p_y+i][p->p_x+plus_droite+1] != 0) ) )
            return 0;
    }
    return 1;
}

/*
 * peut_ici: renvoie 1 si la piece peut etre ici, 0 sinon
 *           ***** APPELLE UNIQUEMENT PAR REFLECHIT *****
 */

inline int peut_ici(int joueur, int x, int y, int forme)
{
    int     i;

    for (i = 0; i < 16; i++)
        if ( ((forme & (1 << i)) != 0) &&
            ( (x+x_inc[i] < x_min[team[joueur]])
            || (x+x_inc[i] > x_max[team[joueur]])
            || (y+y_inc[i] > y_max[team[joueur]])
            || (etat[y+y_inc[i]][x+x_inc[i]] != 0) ) )
            return 0;
    return 1;
}

/*
 * reflechit: l'ordinateur choisit la meilleur colonne et orientation
 */

void reflechit(int joueur, t_piece *p)
{
    int         rotation, i, j, k, l;
    int         ordonnee_piece[4][ETAT_X_MAX];
    int         hauteur_piece[4];
    int         ligne_piece_non_vide[4];
    int         hauteur_maxi[4][ETAT_X_MAX];
    int         nblig[4][ETAT_X_MAX];
    int         nbtrous[4][ETAT_X_MAX];
    int         ligne_complete;
    int         priorite, best_priorite, best_col, best_rotation;
    int         hauteur_inquietante;
    int         ancienne_forme;
    signed char save_etat[16];


    /* Recherche le carre le plus haut de l'aire de jeu */
    j = y_min[team[joueur]];
    k = -1; /* ligne la plus haute */
    while ( (k == -1) && (j <= y_max[team[joueur]]) )
    {
        for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
            if (etat[j][i] > 0)
                k = j;
        j++;
    }
    /* Si l'ordinateur a atteint la moitie de l'aire en hauteur (sur au  */
    /* moins une colonne), on considere que la hauteur est "inquietante" */
    /* (voir plus bas ce que cela change)                                */
    if (k == -1)
        hauteur_inquietante = 0;
    else
        hauteur_inquietante = (k <= ((y_min[team[joueur]]+y_max[team[joueur]])/2));


    /* Pour chaque orientation de piece */
    for (rotation = 0; rotation < 4; rotation++)
    {
        /* Calcule la hauteur de la piece dans cette rotation */
        ligne_piece_non_vide[0] = 0;
        ligne_piece_non_vide[1] = 0;
        ligne_piece_non_vide[2] = 0;
        ligne_piece_non_vide[3] = 0;
        for (k = 0; k < 16; k++)
            if ((p->p_forme & (1 << k)) != 0)
                ligne_piece_non_vide[y_inc[k]] |= 1;
        hauteur_piece[rotation] = ligne_piece_non_vide[0] +
                                  ligne_piece_non_vide[1] +
                                  ligne_piece_non_vide[2] +
                                  ligne_piece_non_vide[3];

        /* Reflexion... */
        for (i = x_min[team[joueur]]-3; i <= x_max[team[joueur]]; i++)
            if (peut_ici(joueur, i, p->p_y+1, p->p_forme))
            {
                j = p->p_y+2;
                while (peut_ici(joueur, i, j, p->p_forme)) j++;
                j--;
                /*ordonnee_piece[rotation][i] = j;*/
                ordonnee_piece[rotation][i] = j-y_min[team[joueur]];
                for (k = 0; k < 16; k++)
                    save_etat[k] = etat[j+y_inc[k]][i+x_inc[k]];
                for (k = 0; k < 16; k++)
                    if ((p->p_forme & (1 << k)) != 0)
                        etat[j+y_inc[k]][i+x_inc[k]] = 1;
                hauteur_maxi[rotation][i] = y_max[team[joueur]];
                nblig[rotation][i] = 0;
                nbtrous[rotation][i] = 0;
                for (k = y_max[team[joueur]]; k >= y_min[team[joueur]]; k--)
                {
                    ligne_complete = 1;
                    for (l = x_min[team[joueur]]; l <= x_max[team[joueur]]; l++)
                    {
                        if (etat[k][l] <= 0)
                        {
                            ligne_complete = 0;
                            /*if ( (k > y_min[joueur]) && (etat[k-1][l] != 0) )*/
                            if (etat[k-1][l] != 0)
                            {
                                nbtrous[rotation][i]++;
                                /*if ( (l == x_min[joueur]) || (etat[k][l-1] != 0) )*/
                                /*if (etat[k][l-1] != 0)
                                    nbtrous[rotation][i]++;*/
                                /*if ( (l == x_max[joueur]) || (etat[k][l+1] != 0) )*/
                                /*if (etat[k][l+1] != 0)
                                    nbtrous[rotation][i]++;*/
                            }
                            /*else
                                if (etat[k-2][l] != 0)
                                    nbtrous[rotation][i]++;
                                else
                                    if (etat[k-3][l] != 0)
                                        nbtrous[rotation][i]++;*/

                            /*if ( ( (l == x_min[joueur]) && (etat[k][l+1] != 0) )
                                || ( (l == x_max[joueur]) && (etat[k][l-1] != 0) )
                                || ( (etat[k][l-1] != 0) && (etat[k][l+1] != 0) ) )
                                nbtrous[rotation][i]++;*/
                        }
                        else
                            if (k < hauteur_maxi[rotation][i])
                                hauteur_maxi[rotation][i] = k;
                    }
                    if (ligne_complete) nblig[rotation][i]++;
                }
                hauteur_maxi[rotation][i] -= y_min[team[joueur]];
                for (k = 0; k < 16; k++)
                    if ((p->p_forme & (1 << k)) != 0)
                        etat[j+y_inc[k]][i+x_inc[k]] = save_etat[k];
            }
            else
                nblig[rotation][i] = -1;
        retourne_piece(255, p);
    }

    best_priorite = -999999999;

    best_col = 0;       /* pour eviter un warning en compilant avec -O2 */
    best_rotation = 0;  /* pour eviter un warning en compilant avec -O2 */

    /* Si l'ordi attei<nt cette hauteur, sa priorite No 1 est de faire des */
    /* lignes au lieu de ne pas faire de trous                             */
    if (hauteur_inquietante)
    {
        for (rotation = 0; rotation < 4; rotation++)
            for (i = x_max[team[joueur]]; i >= x_min[team[joueur]]-3; i--)
                if (nblig[rotation][i] != -1)
                {
                    /*priorite = (nblig[rotation][i] << 10)+
                        (hauteur_maxi[rotation][i]*5)+
                        5000-(nbtrous[rotation][i]*25)+
                        (ordonnee_piece[rotation][i]*30)+
                        4-hauteur_piece[rotation];*/

                    /* hauteur inquietante: la priorite est aux lignes */

                    /* P1: bits 25-27 (0.....7): nombre de lignes */
                    /* P2: bits 14-24 (0..2047): nombre de trous  */
                    /* P3: bits  8-13 (0....63): hauteur maxi     */
                    /* P4: bits  2-7  (0....63): ordonnee piece   */
                    /* P5: bits  0-1  (0.....3): hauteur piece    */

                    priorite = (nblig[rotation][i] << 27) +
                               ((2047-nbtrous[rotation][i]) << 24)+
                               (hauteur_maxi[rotation][i] << 13) +
                               (ordonnee_piece[rotation][i] << 7)+
                               (4-hauteur_piece[rotation]);

                    if (priorite >= best_priorite)
                    {
                        best_priorite = priorite;
                        best_col = i;
                        best_rotation = rotation;
                    }
                }
    }
    else
    {
        for (rotation = 0; rotation < 4; rotation++)
            for (i = x_max[team[joueur]]; i >= x_min[team[joueur]]-3; i--)
                if (nblig[rotation][i] != -1)
                {
                    /* hauteur non inquietante: la priorite est de ne pas faire */
                    /* de trous                                                 */

                    /* P1: bits 17-27 (0..1023): nombre de trous  */
                    /* P2: bits 14-16 (0.....7): nombre de lignes */
                    /* P3: bits  8-13 (0....63): hauteur maxi     */
                    /* P4: bits  2-7  (0....63): ordonnee piece   */
                    /* P5: bits  0-1  (0.....3): hauteur piece    */

                    priorite = ((2047-nbtrous[rotation][i]) << 27)+
                               (nblig[rotation][i] << 16) +
                               (hauteur_maxi[rotation][i] << 13) +
                               (ordonnee_piece[rotation][i] << 7)+
                               (4-hauteur_piece[rotation]);

                    if (priorite >= best_priorite)
                    {
                        best_priorite = priorite;
                        best_col = i;
                        best_rotation = rotation;
                    }
                }
    }
    bonne_colonne[joueur] = best_col;
    ancienne_forme = p->p_forme;
    for (rotation = 0; rotation < best_rotation; rotation++)
        retourne_piece(255, p);
    bonne_forme[joueur] = p->p_forme;
    p->p_forme = ancienne_forme;
}

/*
 * nouvelle_piece: tire au sort une nouvelle piece pour le joueur
 */

void nouvelle_piece(int joueur, int ordi_reflechit)
{
    int     form, form_next, form_next2;

    if (same_pieces)
    {
        form = forme_avance[numero_forme[joueur]];
        numero_forme[joueur] = (numero_forme[joueur]+1)%NB_FORMES_AV;
        form_next = forme_avance[numero_forme[joueur]];
        form_next2 = forme_avance[(numero_forme[joueur]+1)%NB_FORMES_AV];
    }
    else
    {
        form = next_forme[joueur];
        form_next = next_forme2[joueur];
        form_next2 = choisit_piece_hasard(); /*rand()%(7+eccentric*9);*/
        next_forme[joueur] = form_next;
        next_forme2[joueur] = form_next2;
    }
    piece[joueur].p_forme = formes[form];
    piece[joueur].p_bonus = numero_piece_bonus[form];
    next_piece[joueur].p_forme = formes[form_next];
    next_piece[joueur].p_bonus = numero_piece_bonus[form_next];
    next_piece2[joueur].p_forme = formes[form_next2];
    next_piece2[joueur].p_bonus = numero_piece_bonus[form_next2];
    #ifndef MSDOS
    if (X_Window)
    {
        piece[joueur].p_couleur = X_win_couleur_piece[form];
        next_piece[joueur].p_couleur = X_win_couleur_piece[form_next];
        next_piece2[joueur].p_couleur = X_win_couleur_piece[form_next2];
    }
    else
    #endif
    {
        piece[joueur].p_couleur = couleur_piece[form];
        next_piece[joueur].p_couleur = couleur_piece[form_next];
        next_piece2[joueur].p_couleur = couleur_piece[form_next2];
    }
    piece[joueur].p_taille = tc;
    next_piece[joueur].p_taille = tc_next;
    next_piece2[joueur].p_taille = tc_next2;

    if (get_game_team())
        /*  attention pour plusieurs equipes ("*joueur" !!) */
        piece[joueur].p_x = x_min[team[joueur]]+
            ((x_max[team[joueur]]-x_min[team[joueur]]-4*nbj)/(nbj+1)+4)*joueur+4;
    else
        piece[joueur].p_x = x_min[team[joueur]]+1+(x_max[team[joueur]]-x_min[team[joueur]]-4)/2;
    next_piece[joueur].p_x = x_next[joueur]+3;
    next_piece2[joueur].p_x = x_next2[joueur]+3;

    piece[joueur].p_y = y_min[team[joueur]];
    next_piece[joueur].p_y = y_next[joueur]+2;
    next_piece2[joueur].p_y = y_next2[joueur]+2;

    memcpy(&piece_old[joueur], &piece[joueur], sizeof(t_piece));

    if ( (en_jeu) && (view_next > 0) )
    {
        copy_v2s_xy(VSCR_VSCREEN, x_next[joueur]+3, y_next[joueur]+2, tc_next*4, tc_next*4);
        affiche_piece_xy(ECRAN_REEL, joueur, &next_piece[joueur]);
        if (view_next == 2)
        {
            copy_v2s_xy(VSCR_VSCREEN, x_next2[joueur]+3, y_next2[joueur]+2,
                tc_next2*4, tc_next2*4);
            affiche_piece_xy(ECRAN_REEL, joueur, &next_piece2[joueur]);
        }
    }

    if ( (ordi[joueur]) && (ordi_reflechit) )
        reflechit(joueur, &piece[joueur]);
}

/*
 * explosion_ligne: fait exploser une ligne d'un joueur
 */

void explosion_ligne(int joueur, int ligne)
{
    int     i;

    explosions[joueur][nb_explosions[joueur]] = ligne;
    nb_explosions[joueur]++;
    if (ecran_inverse[team[joueur]])
        for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
            putsprite_tr_fond(VSCR_AUTRES+2, &explosion[piece[joueur].p_taille-TC_MIN],
                x_fin[team[joueur]]-(i-x_min[team[joueur]]+1)*piece[joueur].p_taille,
                y_fin[team[joueur]]-(ligne-y_min[team[joueur]]+1)*piece[joueur].p_taille);
    else
        for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
            putsprite_tr_fond(VSCR_AUTRES+2, &explosion[piece[joueur].p_taille-TC_MIN],
                x_deb[team[joueur]]+(i-x_min[team[joueur]])*piece[joueur].p_taille,
                y_deb[team[joueur]]+(ligne-y_min[team[joueur]])*piece[joueur].p_taille);
}

/*
 * pose_piece: pose la piece du joueur et tire au sort une nouvelle
 */

void pose_piece(int joueur)
{
    int     i, j;
    int     old_nb_lignes, nb_lig_explose, ligne_complete[ETAT_Y_MAX];
    int     old_x;

    old_x = piece[joueur].p_x;

    if (get_game_team())
        for (i = 0; i < nbj; i++)
            if ( (i != joueur) && (!perdu[team[i]]) )
                annule_etats(i, &piece[i]);

    etats_positifs(joueur, &piece[joueur]);

    old_nb_lignes = nb_lignes[team[joueur]][0];

    nb_lig_explose = 0;
    for (i = y_max[team[joueur]]; i >= y_min[team[joueur]]; i--)
    {
        ligne_complete[i] = 1;
        for (j = x_min[team[joueur]]; j <= x_max[team[joueur]]; j++)
            /*ligne_complete[i] = ligne_complete[i] & (etat[i][j] != 0);*/
            if (etat[i][j] == 0)
            {
                ligne_complete[i] = 0;
                break;
            }
        if (ligne_complete[i])
        {
            explosion_ligne(joueur, i);
            nb_lig_explose++;
        }
    }

    if ( (nb_lig_explose == 4) && (game_mode != GAME_CHALLENGE) )
    {
        if (langue == ANGLAIS)
        {
            switch (rand()%5)
            {
                case 0:ajoute_msg(joueur, 1, "Good !"); break;
                case 1:ajoute_msg(joueur, 1, "Excellent !"); break;
                case 2:ajoute_msg(joueur, 1, "Well done !"); break;
                case 3:ajoute_msg(joueur, 1, "Beautiful !"); break;
                case 4:ajoute_msg(joueur, 1, "Splendid !"); break;
            }
        }
        else
        {
            switch (rand()%5)
            {
                case 0:ajoute_msg(joueur, 1, "Bien !"); break;
                case 1:ajoute_msg(joueur, 1, "Excellent !"); break;
                case 2:ajoute_msg(joueur, 1, "Bien joue !"); break;
                case 3:ajoute_msg(joueur, 1, "Joli !"); break;
                case 4:ajoute_msg(joueur, 1, "Splendide !"); break;
            }
        }
    }

    /*  traitements son, envoi par reseau, ...  */
    if ( (sb16) && (sons == SOUND_SB16) )
    {
        if (nb_lig_explose == 4)
            SON_EXPLOSION_TETRIS;
        else
            if (nb_lig_explose > 0)
                SON_EXPLOSION;
    }

    if (get_game_team())
        for (i = 0; i < nbj; i++)
            if ( (i != joueur) && (!perdu[team[i]]) )
                etats_negatifs(i, &piece[i]);

    /* stats par piece posee */
    nb_posees[team[joueur]][piece[joueur].p_couleur]++;
    if (get_stats())
        game_write_xy_s_c(ECRAN_REEL, x_fin[team[joueur]]+4*tc_stats+8,
            y_fin[team[joueur]]-((7-piece[joueur].p_couleur)*(3*tc_stats))+tc_stats,
            -15, 72, 1, "%d", nb_posees[team[joueur]][piece[joueur].p_couleur]);

    if (nb_lig_explose > 0)
    {
        if (game_mode == GAME_CHALLENGE)
        {
            nb_lignes[team[joueur]][0] -= nb_lig_explose;
            if (nb_lignes[team[joueur]][0] < 0)
                nb_lignes[team[joueur]][0] = 0;
        }
        else
            nb_lignes[team[joueur]][0] += nb_lig_explose;
        nb_lignes[team[joueur]][nb_lig_explose]++;
        game_write_xy_s_c(ECRAN_REEL, x_fin[team[joueur]]+6+4*tc_next2,
            y_carres_lignes[team[joueur]][nb_lig_explose-1], -15, 98, 1,
            "%d", nb_lignes[team[joueur]][nb_lig_explose]);
        affiche_nombre_lignes(joueur);
    }

    if (nb_lig_explose == 0)
    {
        nouvelle_piece(joueur, 1);
        if (peut_afficher(joueur, &piece[joueur]))
        {
            affiche_piece(ECRAN_REEL, joueur, &piece[joueur]);
            etats_negatifs(joueur, &piece[joueur]);
        }
        else
            joueur_ou_team_perd(joueur);
    }
    else
    {
        if (network)
        {
            switch (nb_lig_explose)
            {
                case 1:
                    if (bascule_envoi[0])
                    {
                        envoye[team[joueur]]++;
                        envoi_lignes(1);
                    }
                    bascule_envoi[0] ^= 1;
                    break;
                case 2:
                case 3:
                    envoye[team[joueur]] += nb_lig_explose-1;
                    envoi_lignes(nb_lig_explose-1);
                    break;
                case 4:
                    envoye[team[joueur]] += 4;
                    envoi_lignes(4);
                    break;
            }
        }
        else
        {
            for (i = 0; i < nbj; i++)
                if ( (!perdu[team[i]]) && (i != joueur) && (team[i] != team[joueur]) )
                {
                    if (nbc > 0)
                        switch (nb_lig_explose)
                        {
                            case 1:
                                if (bascule_envoi[team[joueur]])
                                    doit_remonter[team[i]]++;
                                break;
                            case 2:
                            case 3:
                                doit_remonter[team[i]] += nb_lig_explose-1;
                                break;
                            case 4:
                                doit_remonter[team[i]] += 4;
                                if (ecran_inverse[team[i]] == 0)
                                {
                                    ecran_inverse[team[i]] = 1;
                                    redessine_aire_jeu(i);
                                }
                                break;
                        }
                    else
                        switch (nb_lig_explose)
                        {
                            case 2:
                            case 3:
                                doit_remonter[team[i]] += nb_lig_explose-1;
                                break;
                            case 4:
                                doit_remonter[team[i]] += 4;
                                if (ecran_inverse[team[i]] == 0)
                                {
                                    ecran_inverse[team[i]] = 1;
                                    redessine_aire_jeu(i);
                                }
                                break;
                        }
                }
        }
        if (nbc > 0)
            switch (nb_lig_explose)
            {
                case 1:
                    bascule_envoi[team[joueur]] ^= 1;
                    if (!bascule_envoi[team[joueur]])
                        envoye[team[joueur]]++;
                    break;
                case 2:
                case 3:
                    envoye[team[joueur]] += nb_lig_explose-1;
                    break;
                case 4:
                    envoye[team[joueur]] += 4;
                    break;
            }
        else
            switch (nb_lig_explose)
            {
                case 2:
                case 3:
                    envoye[team[joueur]] += nb_lig_explose-1;
                    break;
                case 4:
                    envoye[team[joueur]] += 4;
                    break;
            }

        /* incrementation niveau si un seul joueur humain (sans reseau) */
        if ( (!network) && (nbp == 1) && (nbc == 0)
            && (game_mode != GAME_CHALLENGE)
            && ( (nb_lignes[team[joueur]][0] / 10) != (old_nb_lignes / 10) ) )
        {
            i = nb_lignes[team[joueur]][0] / 10;
            if (i <= NIVEAU_MAX)
                change_niveau(i);
        }

        /* entree dans niveau bonus toutes les 15 lignes */
        if ( (get_bonus_level())
            && ((nb_lignes[team[joueur]][0] / 15) != (old_nb_lignes / 15)) )
            /* POUR TEST : */
            /*&& ((nb_lignes[team[joueur]][0] / 2) != (old_nb_lignes / 2)) )*/
        {
            /*  */
            bonus_prevu[joueur] = get_bonus_level();
            if (nb_lig_explose == 4)
                bonus_voix_son[joueur] = 4;
            else
                bonus_voix_son[joueur] = 5;
        }
    }
    envoye[team[joueur]] += nb_lig_explose;

    /* si c'est un joueur humain qui pose la piece et qu'il est en equipe    */
    /* avec au moins un ordinateur, on fait re-reflechir tous les ordis...   */
    /*  attention si plusieurs equipes, tenir compte seulement du/des */
    /* ordi(s) de l'equipe et non pas "nbc"                                  */
    if ( /*(!ordi[joueur]) &&*/ (nbc > 0) && (get_game_team()) )
    {
        for (i = 0; i < nbj; i++)
            if ( (ordi[i]) && (bonne_colonne[i] >= old_x)
                && (bonne_colonne[i] <= old_x+3) )
                reflechit(i, &piece[i]);
    }
}

/*
 * finit_pose_piece: finit la pose d'une piece (nouvelle piece + affichage complet de l'aire de jeu)
 *                   Cette fonction est appelee apres une explosion seulement
 */

void finit_pose_piece(int joueur)
{
    int     i, j, k, ligne_max;

    if (get_game_team())
        for (i = 0; i < nbj; i++)
            if ( (i != joueur) && (!perdu[team[i]]) )
                annule_etats(i, &piece[i]);

    for (i = 0; i < nb_explosions[joueur]; i++)
    {
        for (j = explosions[joueur][i]; j > y_min[team[joueur]]; j--)
            for (k = x_min[team[joueur]]; k <= x_max[team[joueur]]; k++)
                etat[j][k] = etat[j-1][k];
        for (j = i+1; j < nb_explosions[joueur]; j++)
            explosions[joueur][j]++;
    }
    for (k = x_min[team[joueur]]; k <= x_max[team[joueur]]; k++)
        etat[y_min[team[joueur]]][k] = 0;

    if (get_game_team())
        for (i = 0; i < nbj; i++)
            if ( (i != joueur) && (!perdu[team[i]]) )
                etats_negatifs(i, &piece[i]);

    if (ecran_inverse[team[joueur]])
    {
        ecran_inverse[team[joueur]] = 0;
        redessine_aire_jeu(joueur);
    }
    else
    {
        ligne_max = explosions[joueur][0];
        for (i = x_min[team[joueur]]; i <= x_max[team[joueur]]; i++)
            for (j = y_min[team[joueur]]; j <= ligne_max; j++)
                if ( (abs(etat[j][i]) == ETAT_PIECE_BONUS) || (etat[j][i] == 0) )
                    game_efface_brique(ECRAN_REEL, joueur, i-x_min[team[joueur]],
                        j-y_min[team[joueur]], piece[joueur].p_taille);
                else
                    game_affiche_brique(ECRAN_REEL, joueur,
                        i-x_min[team[joueur]], j-y_min[team[joueur]],
                        piece[joueur].p_taille, abs(etat[j][i])-1);
    }

    nb_explosions[joueur] = 0;
    aff_explosions[joueur] = 0;

    nouvelle_piece(joueur, 1);

    if (peut_afficher(joueur, &piece[joueur]))
    {
        if (bonus_prevu[joueur] != NO_BONUS)
        {
            bonus[joueur] = bonus_prevu[joueur];
            bonus_prevu[joueur] = NO_BONUS;
            init_bonus(joueur);
        }
        else
        {
            if ( (game_mode == GAME_CHALLENGE) && (nb_lignes[0][0] == 0) )
            {
                if (niveau == NIVEAU_MAX)
                {
                    partie_finie = 1;
                    coul_winner = COUL_1_WINNER;
                    dir_coul_winner = 1;
                }
                else
                {
                    change_niveau(niveau+1);
                    challenge_init_niveau();
                }
            }
            else
            {
                affiche_piece(ECRAN_REEL, joueur, &piece[joueur]);
                etats_negatifs(joueur, &piece[joueur]);
            }
        }
    }
    else
        joueur_ou_team_perd(joueur);
}

/*
 * boucle_jeu: jeu proprement dit
 */

void boucle_jeu()
{
    #ifndef MSDOS
    /*struct itimerval    timer1, timer2;*/
    #endif
    int                 i, j, k;
    signed char         etat_sav[ETAT_Y_MAX][ETAT_X_MAX];   /* sauvegarde de etats */
    t_piece             piece_tmp;

    /*init_aires_de_jeu();*/
    if (!en_pause)
        init_jeu();

    for (i = 0; i < 15; i++)
        setcoul(160+i, pal_ftris[(160+i)*3], pal_ftris[(160+i)*3+1],
            pal_ftris[(160+i)*3+2]);

    memcpy(etat_sav, etat, ETAT_X_MAX*ETAT_Y_MAX);

    /* restaure les variables sauvees avant la pause */
    if (en_pause)
    {
        memcpy(etat, etat_pause, ETAT_X_MAX*ETAT_Y_MAX);
        for (i = 0; i < nbj; i++)
        {
            memcpy(&piece[i], &piece_pause[i], sizeof(t_piece));
            next_forme[i] = next_forme_pause[i];
            next_forme2[i] = next_forme2_pause[i];
            memcpy(&next_piece[i], &next_piece_pause[i], sizeof(t_piece));
            memcpy(&next_piece2[i], &next_piece2_pause[i], sizeof(t_piece));
            memcpy(&piece_old[i], &piece_old_pause[i], sizeof(t_piece));
            numero_forme[i] = numero_forme_pause[i];
        }
        for (i = 0; i < NB_FORMES_AV; i++)
            forme_avance[i] = forme_avance_pause[i];
    }

    for (i = 0; i < nbj; i++)
        if ( (i == 0) || (team[i] != team[i-1]) )
            copy_v2v_xy(VSCR_AUTRES+2, VSCR_VSCREEN, x_deb[team[i]],
                y_deb[team[i]], x_fin[team[i]]-x_deb[team[i]],
                y_fin[team[i]]-y_deb[team[i]]);

    /*if ( (en_pause) && (view_next > 0) )
        for (i = 0; i < nbj; i++)
        {
            copy_v2v_xy(VSCR_AUTRES+2, VSCR_VSCREEN, x_fin[i]+3, y_deb[i],
                4*tc_next+4, 4*tc_next+4);
            if (view_next == 2)
                copy_v2v_xy(VSCR_AUTRES+2, VSCR_VSCREEN, x_fin[i]+3, y_deb[i]+4*tc_next+7,
                    4*tc_next2+4, 4*tc_next2+4);
        }*/

    if (!en_pause)
        copy_v2v(VSCR_VSCREEN, VSCR_JEU_VIDE);
    else
        copy_v2v(VSCR_JEU_VIDE, VSCR_VSCREEN);

    if (!en_pause)
        niveau = 0;

    quitte_jeu = 0;
    en_jeu = 1;

    if (!en_pause)
    {
        switch (get_time_limite())
        {
            case 1:cptr_time = 0; break;
            case 2:cptr_time = 60; break;
            case 3:cptr_time = 60*2; break;
            case 4:cptr_time = 60*5; break;
            case 5:cptr_time = 60*10; break;
        }
        time_change = 1;
        for (i = 0; i < nbj; i++)
        {
            bonus_timer_ready[i] = 0;
            bonus_demarre[i] = 0;
            bonus_prevu[i] = NO_BONUS;
            bonus[i] = NO_BONUS;
            brix_niveau[i] = 0;
            ecran_inverse[team[i]] = 0;
            perdu[team[i]] = 0;
            for (j = 0; j < 5; j++)
                nb_lignes[team[i]][j] = 0;
            nb_explosions[i] = 0;
            aff_explosions[i] = 0;
            for (j = 0; j < 7; j++)
                nb_posees[team[i]][j] = 0;
            /*write_xy_c_f(0, i*font_height, 4, 0, "%02d", nb_lignes[i]);*/
            descend[i] = 0; descend_cptr[i] = 0; rotation_cptr[i] = 0;
            doit_remonter[team[i]] = 0;
            envoye[team[i]] = 0;
            bascule_envoi[team[i]] = 0;
            msg[i][0][0] = '\0';
            msg[i][1][0] = '\0';
            cptr_msg[i] = 0;
            cptr_msg[i] = 0;
            nouvelle_piece(i, 1);
            affiche_piece(ECRAN_REEL, i, &piece[i]);
            etats_negatifs(i, &piece[i]);
            affiche_nombre_lignes(team[i]);
        }
        tous_perdu = 0;
        partie_finie = 0;
    }

    if ( (!en_pause) && (!network) && (nbp == 1) && (nbc == 0) )
    {
        change_niveau(0);
        if (game_mode == GAME_CHALLENGE)
            challenge_init_niveau();
    }

    if (sb16)
    {
        if ( (!en_pause) && (music_menu != music_jeu) )
            change_music(NULL, 1);
        else
        {
            ftris_vide_samples();
            ftris_charge_samples(1);
        }
    }

    if ( (sb16) && (status_jeu) )
    {
        affiche_volume_jeu(ECRAN_REEL);
        affiche_musique_jeu(ECRAN_REEL);
    }

    en_pause = 0;

    #ifndef MSDOS
    detourne_int_timer();
    #endif

    #ifndef MSDOS
    if (!X_Window)
        keyboard_update();
    #endif

    game_affiche_pos_musique(ECRAN_REEL, 1);

    /* POUR TEST : */
    /*nb_lignes[0][0] = 138;
    niveau = 13;*/
    while (!quitte_jeu)     /* BOUCLE PRINCIPALE */
    {
        #ifndef MSDOS
        nouveau_timer();
        #endif

        if (module_fini)
            change_music(NULL, 1);

        image_bombe = (image_bombe+1)%20;

        if (partie_finie)
        {
            coul_winner += dir_coul_winner;
            if ( (coul_winner == COUL_1_WINNER)
                || (coul_winner == COUL_2_WINNER) )
                dir_coul_winner = -dir_coul_winner;
            if (network)
            {
                if (!tous_perdu)
                {
                    for (i = 0; i < nbj; i++)
                        if (langue == ANGLAIS)
                            write_xy_c_o(ECRAN_REEL,
                                x_deb[i]+(x_fin[i]-x_deb[i]-56)/2,
                                y_deb[i]+(y_fin[i]-y_deb[i]-8)/2,
                                coul_winner, 1, 29, "WINNER!");
                        else
                            write_xy_c_o(ECRAN_REEL,
                                x_deb[i]+(x_fin[i]-x_deb[i]-48)/2,
                                y_deb[i]+(y_fin[i]-y_deb[i]-8)/2,
                                coul_winner, 1, 29, "GAGNE!");
                }
            }
            else
            {
                for (i = 0; i < nbj; i++)
                    if (!perdu[team[i]])
                    {
                        if (game_mode == GAME_CHALLENGE)
                        {
                            write_xy_c_o(ECRAN_REEL,
                                    x_deb[team[i]]+(x_fin[team[i]]-x_deb[team[i]]-72)/2-4,
                                    y_deb[team[i]]+(y_fin[team[i]]-y_deb[team[i]]-8)/2-10,
                                    coul_winner, 1, 29, "CHALLENGE");
                            if (langue == ANGLAIS)
                                write_xy_c_o(ECRAN_REEL,
                                    x_deb[team[i]]+(x_fin[team[i]]-x_deb[team[i]]-56)/2,
                                    y_deb[team[i]]+(y_fin[team[i]]-y_deb[team[i]]-8)/2,
                                    coul_winner, 1, 29, "WINNER!");
                            else
                                write_xy_c_o(ECRAN_REEL,
                                    x_deb[team[i]]+(x_fin[team[i]]-x_deb[team[i]]-48)/2,
                                    y_deb[team[i]]+(y_fin[team[i]]-y_deb[team[i]]-8)/2,
                                    coul_winner, 1, 29, "GAGNE!");
                        }
                        else
                        {
                            if (langue == ANGLAIS)
                                write_xy_c_o(ECRAN_REEL,
                                    x_deb[team[i]]+(x_fin[team[i]]-x_deb[team[i]]-56)/2,
                                    y_deb[team[i]]+(y_fin[team[i]]-y_deb[team[i]]-8)/2,
                                    coul_winner, 1, 29, "WINNER!");
                            else
                                write_xy_c_o(ECRAN_REEL,
                                    x_deb[team[i]]+(x_fin[team[i]]-x_deb[team[i]]-48)/2,
                                    y_deb[team[i]]+(y_fin[team[i]]-y_deb[team[i]]-8)/2,
                                    coul_winner, 1, 29, "GAGNE!");
                        }
                    }
            }
        }

        if (!partie_finie)
            for (i = 0; i < nbj; i++)
                if ( (bonus[i] == NO_BONUS) && (!perdu[team[i]]) && (descend[i])
                    && (nb_explosions[i] == 0)
                    && (aff_explosions[i] == 0)
                    && (!challenge_ready_affiche) )
                    {
                        descend[i] = 0;
                        switch (peut_descendre(i, &piece[i]))
                        {
                            case 0: /* OK pour descente */
                                annule_etats(i, &piece[i]);
                                piece[i].p_y++;
                                etats_negatifs(i, &piece[i]);
                                break;
                            case 1: /* mur ou bas ecran */
                                pose_piece(i);
                                if (!ordi[i])
                                {
                                    #ifdef MSDOS
                                    touche_cptr[i] = 12;
                                    #else
                                    /*touche_cptr[i] = 6;*/
                                    touche_cptr[i] = 12;
                                    #endif
                                }
                                break;
                            case 2: /* autre joueur */
                                descend_cptr[i] = 0;
                                break;
                        }
                    }

        waitretrace();
        /* affichage des pieces de tous les joueurs */
        if (!partie_finie)
            for (i = 0; i < nbj; i++)
                if (!perdu[team[i]])
                {
                    if (bonus[i] == NO_BONUS)
                    {
                        if (nb_explosions[i] == 0)
                        {
                            if ( (piece[i].p_bonus != 0)
                                || (memcmp(&piece_old[i], &piece[i], sizeof(t_piece)) != 0) )
                            {
                                annule_etats(i, &piece_old[i]);
                                efface_piece(ECRAN_REEL, i, &piece_old[i]);
                                affiche_piece(ECRAN_REEL, i, &piece[i]);
                                memcpy(&piece_old[i], &piece[i], sizeof(t_piece));
                                etats_negatifs(i, &piece[i]);
                            }
                        }
                        else
                        {
                            aff_explosions[i]++;
                            if (aff_explosions[i] == 4)
                                finit_pose_piece(i);
                        }
                    }
                    if ( (challenge_ready_affiche)
                        && (challenge_timer_ready == 0) )
                    {
                        challenge_copie_ecran_apres_ready();
                        challenge_ready_affiche = 0;
                    }
                    if ( (cptr_msg[i] > 0) && (get_window_msg()) )
                    {
                        cptr_msg[i]--;
                        if (cptr_msg[i] == 0)
                            efface_msgs(i);
                    }
                }
        #ifndef MSDOS
        if (X_Window)
            XSync(display, 0);
        #endif
        if (!partie_finie)
            for (i = 0; i < nbj; i++)
                if ( (!perdu[team[i]]) && (!challenge_ready_affiche) )
                {
                    if (bonus[i] != NO_BONUS)
                    {
                        if (bonus_demarre[i])
                            traite_bonus(i);
                        else
                            if (bonus_timer_ready[i] == 0)
                            {
                                copy_v2s_xy(VSCR_AUTRES+3, x_deb[team[i]],
                                    y_deb[team[i]], x_fin[team[i]]-x_deb[team[i]],
                                    y_fin[team[i]]-y_deb[team[i]]);
                                bonus_demarre[i] = 1;
                            }
                    }
                    else
                        if ( (nb_explosions[i] == 0)
                            && (aff_explosions[i] == 0) )
                        {
                            if (doit_remonter[team[i]] > 0)
                            {
                                memcpy(&piece_tmp, &piece[i], sizeof(t_piece));
                                piece_tmp.p_y++;
                                if ( (peut_descendre(i, &piece[i]) == 0)
                                    && (peut_descendre(i, &piece_tmp) == 0) )
                                {
                                    doit_remonter[team[i]]--;
                                    annule_etats_team(i);
                                    for (j = y_min[team[i]]; j <= y_max[team[i]]; j++)
                                        for (k = x_min[team[i]]; k <= x_max[team[i]]; k++)
                                            etat[j][k] = etat[j+1][k];
                                    for (k = x_min[team[i]]; k <= x_max[team[i]]; k++)
                                        etat[y_max[team[i]]][k] = (rand()%7)+1;
                                    etat[y_max[team[i]]][x_min[team[i]]+
                                        (rand()%(x_max[team[i]]-x_min[team[i]]+1))] = 0;
                                    etats_negatifs_team(i);
                                    redessine_aire_jeu(i);
                                }
                            }
                            switch (ordi[i])
                            {
                                case 0:     /* humain */
                                    /* gauche */
                                    if ( (touche_cptr[i] == 0) && (key_tab[touche[nbp-1][i][0]])
                                        && (peut_aller_gauche(i, &piece[i])) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        piece[i].p_x--;
                                        etats_negatifs(i, &piece[i]);
                                        #ifdef MSDOS
                                        touche_cptr[i] = 13;
                                        #else
                                        if (X_Window)
                                            key_tab[touche[nbp-1][i][0]] = 0;
                                        /*touche_cptr[i] = 6;*/
                                        touche_cptr[i] = 9;
                                        #endif
                                    }
                                    /* droite */
                                    if ( (touche_cptr[i] == 0) && (key_tab[touche[nbp-1][i][1]])
                                        && (peut_aller_droite(i, &piece[i])) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        piece[i].p_x++;
                                        etats_negatifs(i, &piece[i]);
                                        #ifdef MSDOS
                                        touche_cptr[i] = 13;
                                        #else
                                        if (X_Window)
                                            key_tab[touche[nbp-1][i][1]] = 0;
                                        /*touche_cptr[i] = 6;*/
                                        touche_cptr[i] = 9;
                                        #endif
                                    }
                                    /* 5 = retouner piece */
                                    if ( (rotation_cptr[i] == 0) && (key_tab[touche[nbp-1][i][3]]) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        retourne_piece(i, &piece[i]);
                                        etats_negatifs(i, &piece[i]);
                                        /*piece[i].p_forme = retourne_piece(piece[i].p_forme);*/
                                        /*key_tab[touche[nbp-1][i][3]] = 0;*/
                                        #ifdef MSDOS
                                        rotation_cptr[i] = 20;
                                        #else
                                        if (X_Window)
                                            key_tab[touche[nbp-1][i][3]] = 0;
                                        /*rotation_cptr[i] = 9;*/
                                        rotation_cptr[i] = 15;
                                        #endif
                                    }
                                    /* 2 = descendre piece */
                                    if ( (touche_cptr[i] == 0) && (key_tab[touche[nbp-1][i][2]]) )
                                    {
                                        switch (peut_descendre(i, &piece[i]))
                                        {
                                            case 0: /* OK pour descente */
                                                descend_cptr[i] = 0; descend[i] = 0;
                                                annule_etats(i, &piece[i]);
                                                piece[i].p_y++;
                                                etats_negatifs(i, &piece[i]);
                                                /*key_tab[touche[nbp-1][i][2]] = 0;*/
                                                #ifdef MSDOS
                                                touche_cptr[i] = 2;
                                                #else
                                                touche_cptr[i] = 2;
                                                #endif
                                                break;
                                            case 1: /* mur ou bas ecran */
                                                pose_piece(i);
                                                #ifdef MSDOS
                                                touche_cptr[i] = 12;
                                                #else
                                                /*touche_cptr[i] = 7;*/
                                                touche_cptr[i] = 12;
                                                #endif
                                                break;
                                            case 2: /* autre joueur */
                                                descend_cptr[i] = 0;
                                                descend[i] = 0;
                                                break;
                                        }
                                        #ifndef MSDOS
                                        if (X_Window)
                                            key_tab[touche[nbp-1][i][2]] = 0;
                                        #endif
                                    }
                                    break;
                                case 1:     /* ordinateur */
                                    if ( (piece[i].p_x > bonne_colonne[i])
                                        && (peut_aller_gauche(i, &piece[i])) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        piece[i].p_x--;
                                        etats_negatifs(i, &piece[i]);
                                    }
                                    if ( (piece[i].p_x < bonne_colonne[i])
                                        && (peut_aller_droite(i, &piece[i])) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        piece[i].p_x++;
                                        etats_negatifs(i, &piece[i]);
                                    }
                                    if ( (piece[i].p_forme != bonne_forme[i])
                                        && (rotation_cptr[i] == 0) )
                                    {
                                        annule_etats(i, &piece[i]);
                                        retourne_piece(i, &piece[i]);
                                        etats_negatifs(i, &piece[i]);
                                        rotation_cptr[i] = 10-2*difficulte;
                                    }
                                    break;
                            }
                        }
                }

        if ( (time_change)
            && (get_time_limite() > 0) && (!challenge_ready_affiche) )
        {
            time_change = 0;
            affiche_temps();
        }

        if ( (game_mode == GAME_CHALLENGE) && (!challenge_ready_affiche) )
        {
            challenge_clignote_lignes = (challenge_clignote_lignes+1)%50;
            if (challenge_clignote_lignes == 0)
                efface_nombre_lignes(0);
            else
                if (challenge_clignote_lignes == 15)
                    affiche_nombre_lignes(0);
        }

        if (network)
            if (net_recv())
                traite_message_recu();

        /* Echap = quitte le jeu */
        if (key_tab[1])
        {
            quitte_jeu = 1;
            if (network)
            {
                for (i = 0; i < nb_PC_net; i++)
                    if ( (i != numero_PC_net) && (PC_net[i].adresse != 0) )
                    {
                        envoi_disconnect(PC_net[i].adresse, PC_net[i].port);
                        PC_net[i].adresse = 0;
                    }
            }
        }

        /* Enter ou Espace si "Ready for challenge ?" affiche */
        if ( (challenge_ready_affiche)
            && ( (key_tab[SCANCODE_ENTER]) || (key_tab[SCANCODE_KEYPADENTER])
                || (key_tab[57]) ) )
        {
            challenge_copie_ecran_apres_ready();
            challenge_ready_affiche = 0;
            challenge_timer_ready = 0;
        }

        /* Enter = quitte le jeu si la partie est finie (acces aux scores) */
        if ( (key_tab[SCANCODE_ENTER] || key_tab[SCANCODE_KEYPADENTER]) && (partie_finie) )
            quitte_jeu = 1;

        /* F10 = pause */
        if ( (key_tab[68]) && (!partie_finie) )
        {
            en_pause = 1;
            quitte_jeu = 1;
        }

        if (sb16)
        {
            /* affiche la barre d'avancement dans la musique */
            game_affiche_pos_musique(ECRAN_REEL, 0);

            /* '+' = augmente le volume */
            if ( (key_tab[78]) && (volume < VOLUME_MAX) )
            {
                if (key_tab[SCANCODE_LEFTCONTROL] || key_tab[SCANCODE_RIGHTCONTROL])
                    volume = (volume > VOLUME_MAX-INC_VOLUME_VITE)?VOLUME_MAX:volume+INC_VOLUME_VITE;
                else
                    volume++;
                setvolume(volume);
                if (status_jeu)
                    affiche_volume_jeu(ECRAN_REEL);
                key_tab[78] = 0;
            }
            /* '-' = diminue le volume */
            if ( (key_tab[74]) && (volume > 0) )
            {
                if (key_tab[SCANCODE_LEFTCONTROL] || key_tab[SCANCODE_RIGHTCONTROL])
                    volume = (volume < INC_VOLUME_VITE)?0:volume-INC_VOLUME_VITE;
                else
                    volume--;
                setvolume(volume);
                if (status_jeu)
                    affiche_volume_jeu(ECRAN_REEL);
                key_tab[74] = 0;
            }
            /* F5 = musique precedente */
            if ( (key_tab[63]) && (music_jeu != MUSIC_RANDOM)
                && (music_jeu > MUSIC1) )
            {
                music_jeu--;
                change_music(NULL, 1);
                if (status_jeu)
                    affiche_musique_jeu(ECRAN_REEL);
                key_tab[63] = 0;
            }
            /* F6 = musique suivante */
            if ( (key_tab[64]) &&
                ( (music_jeu == MUSIC_RANDOM) || (music_jeu < NB_MUSIC) ) )
            {
                if (music_jeu != MUSIC_RANDOM)
                    music_jeu++;
                change_music(NULL, 1);
                if (status_jeu)
                    affiche_musique_jeu(ECRAN_REEL);
                key_tab[64] = 0;
            }
            /* F11 = pattern precedent */
            if (key_tab[87])
            {
                orderpos = (orderpos == 0)?-1:orderpos-2;
                row = 64;
                key_tab[87] = 0;
            }

            /* F12 = pattern suivant */
            if (key_tab[88])
            {
                row = 64;
                key_tab[88] = 0;
            }

            /*  R = "Random" mode pour la musique */
            if (key_tab[19])
            {
                if (music_jeu == MUSIC_RANDOM)
                {
                    music_jeu = music_jouee;
                    music_stop_at_end = 0;
                }
                else
                {
                    music_jeu = MUSIC_RANDOM;
                    music_stop_at_end = 1;
                }
                if (status_jeu)
                    affiche_musique_jeu(ECRAN_REEL);
                game_affiche_pos_musique(ECRAN_REEL, 1);
                key_tab[19] = 0;
            }
        }
        /*  Alt-G = sauve l'image de jeu en RAW */
        if ( (key_tab[56]) && (key_tab[34]) )
        {
            save_graphics_raw();
            key_tab[56] = 0;
            key_tab[34] = 0;
        }
        #ifndef MSDOS
        if (X_Window)
        {
            if (XPending(display) > 0)
                X_traite_evenement();
        }
        else
            keyboard_update();
        #endif
    }
    #ifndef MSDOS
    restaure_int_timer();
    #endif

    if (en_pause)
    {
        memcpy(etat_pause, etat, ETAT_X_MAX*ETAT_Y_MAX);
        for (i = 0; i < nbj; i++)
        {
            memcpy(&piece_pause[i], &piece[i], sizeof(t_piece));
            next_forme_pause[i] = next_forme[i];
            next_forme2_pause[i] = next_forme2[i];
            memcpy(&next_piece_pause[i], &next_piece[i], sizeof(t_piece));
            memcpy(&next_piece2_pause[i], &next_piece2[i], sizeof(t_piece));
            memcpy(&piece_old_pause[i], &piece_old[i], sizeof(t_piece));
            numero_forme_pause[i] = numero_forme[i];
        }
        for (i = 0; i < NB_FORMES_AV; i++)
            forme_avance_pause[i] = forme_avance[i];
        copy_s2v(VSCR_JEU);
    }
    else
        niveau = 0;

    memcpy(etat, etat_sav, ETAT_X_MAX*ETAT_Y_MAX);

    en_jeu = 0;

    if ( (partie_finie) && (!network) && (nbp == 1) && (nbc == 0)
        && (game_mode != GAME_CHALLENGE) )
        high_scores();

    if (sb16)
    {
        if ( (!en_pause) && (music_menu != music_jeu) )
            change_music(NULL, 1);
        else
        {
            ftris_vide_samples();
            ftris_charge_samples(0);
        }
    }
}
