/* ====
   ==== FlashTris Game, (c) 1999-2001 by FlashCode (Sebastien Helleu)
   ====
   ====    web: http://flashcode.free.fr
   ==== e-mail: flashcode@free.fr
   ====
   ==== net.c: fonctions reseau compatibles Dos/Linux
   ====

   ==== This file is part of FlashTris.
   ====
   ==== FlashTris is free software; you can redistribute it and/or modify
   ==== it under the terms of the GNU General Public License as published by
   ==== the Free Software Foundation; either version 2 of the License, or
   ==== (at your option) any later version.
   ====
   ==== FlashTris is distributed in the hope that it will be useful,
   ==== but WITHOUT ANY WARRANTY; without even the implied warranty of
   ==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ==== GNU General Public License for more details.
   ====
   ==== You should have received a copy of the GNU General Public License
   ==== along with FlashTris; if not, write to the Free Software
   ==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@free.fr>
Creation: 1999/06/21

2001/09/06: FlashCode:
    - ajout de la licence
*/


#include <unistd.h>
#include <stdio.h>
#ifndef MSDOS
    #include <netdb.h>
    #include <netinet/in.h>
    #include <sys/ioctl.h>
#endif
/*#include <netinet/in.h>*/
/*#include <sys/types.h> */
/*#include <sys/socket.h>*/
/*#include <sys/param.h> */
#include "net.h"


int                     net_error;          /* code erreur retour d'une fct reseau  */

static unsigned long    udp_addr;           /* adresse locale                       */

int                     udp_socket;         /* socket pour UDP (envoi)              */
int                     udp_socket_read;    /* socket pour recevoir en UDP          */

#ifndef MSDOS
struct sockaddr_in      adresse_tmp;        /* pour stocker l'adresse de l'envoyeur */
#endif


/* ================================== UDP =================================== */

/*
 * udp_open_socket: ouvre un "socket" pour UDP
 *                  Renvoie: > 0 = numero de socket attribue
 *                            -1 si echec
 */

int udp_open_socket(int port)
{
    #ifndef MSDOS
    struct sockaddr_in  adresse_tmp;
    int                 socket_tmp, true;

    if ((socket_tmp = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
        return -1;
    true = 1;
    if (ioctl(socket_tmp, FIONBIO, &true) == -1)
    {
        close(socket_tmp);
        return -1;
    }
    adresse_tmp.sin_family = AF_INET;
    adresse_tmp.sin_addr.s_addr = INADDR_ANY;
    adresse_tmp.sin_port = htons(port);
    if (bind(socket_tmp, (void *)&adresse_tmp, sizeof(adresse_tmp)) == -1)
    {
        close(socket_tmp);
        return -2;
    }
    return socket_tmp;
    #endif
}

/*
 * udp_init: initialise le protocole UDP
 *           Renvoie:  0 si OK
 *                    -1 si erreur init
 *                    -2 si erreur bind (il faut chercher un autre port)
 */

int udp_init(int port, unsigned long *adresse_locale)
{
    unsigned char       nom_machine[500];
    struct hostent      *local;
    int                 ret;

    #ifndef MSDOS
    gethostname(nom_machine, 500);
    local = gethostbyname(nom_machine);
    if (local == NULL)
        local = gethostbyname("127.0.0.1");
    udp_addr = *(int *)local->h_addr_list[0];
    ret = udp_open_socket(0);
    if (ret < 0)
        return -1;
    udp_socket = ret;
    ret = udp_open_socket(port);
    if (ret < 0)
    {
        close(udp_socket);
        return ret;
    }
    udp_socket_read = ret;
    *adresse_locale = udp_addr;
    return 0;
    #else
    /*  */
    return -1;
    #endif
}

/*
 * udp_send: envoie un paquet UDP
 */

int udp_send(unsigned long adresse, int port, int longueur,
             unsigned char *message)
{
    #ifndef MSDOS
    struct sockaddr_in  adresse_tmp;

    adresse_tmp.sin_family = AF_INET;
    adresse_tmp.sin_addr.s_addr = adresse;
    adresse_tmp.sin_port = htons(port);
    if (sendto(udp_socket, message, longueur, 0,
        (struct sockaddr *)&adresse_tmp, sizeof(adresse_tmp)) == -1)
        return 0;
    else
        return 1;
    #endif
}

/*
 * udp_recv: scrute les paquets UDP recus
 *           Renvoie: 0 si aucun paquet recu
 *                    1 si un paquet a ete recu
 */

int udp_recv(int longueur_max, unsigned char *message, int *long_msg_recu,
             unsigned long *adresse_envoyeur)
{
    #ifndef MSDOS
    int                 ret, addrlen;

    addrlen = sizeof(struct sockaddr);
    ret = recvfrom(udp_socket_read, message, longueur_max, 0, &adresse_tmp, &addrlen);
    if (ret <= 0)
        return 0;
    *long_msg_recu = ret;
    if (adresse_envoyeur == NULL)
        return 0;
    *adresse_envoyeur = adresse_tmp.sin_addr.s_addr;
    return 1;
    #endif
}

/*
 * udp_close: ferme UDP
 */

int udp_close()
{
    #ifndef MSDOS
    close(udp_socket);
    close(udp_socket_read);
    #endif
    return 1;
}

/*
 * udp_nom2addr: transforme un nom en une adresse
 */

unsigned long udp_nom2addr(unsigned char *nom_machine)
{
    #ifndef MSDOS
    struct hostent  *host;

    host = gethostbyname(nom_machine);
    if (host == NULL)
        return 0;
    return (*(int *)host->h_addr_list[0]);
    #else
    return 0;
    #endif
}

/* ================================== IPX =================================== */

/*
 * ipx_init: initialise IPX
 */

int ipx_init(int port, unsigned long *adresse_locale)
{
    /*  A DEVELOPPER  */
    return -1;
}

/*
 * ipx_send: envoie un paquet IPX
 */

int ipx_send(unsigned long adresse, int port, int longueur,
             unsigned char *message)
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * ipx_recv: scrute les paquets IPX recus
 */

int ipx_recv(int longueur_max, unsigned char *message, int *long_msg_recu,
             unsigned long *adrese_envoyeur)
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * ipx_close: ferme IPX
 */

int ipx_close()
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * ipx_nom2addr: transforme un nom en une adresse
 */

unsigned long ipx_nom2addr(unsigned char *nom_machine)
{
    /*  A DEVELOPPER  */
    return 0;
}

/* ================================= MODEM ================================== */

/*
 * modem_init: initialise le modem (compose le numero + initialisations)
 */

int modem_init(int port, unsigned long *adresse_locale)
{
    /*  A DEVELOPPER  */
    return -1;
}

/*
 * modem_send: envoie un paquet par modem
 */

int modem_send(unsigned long adresse, int port, int longueur,
               unsigned char *message)
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * modem_recv: scrute les paquets recus par modem
 */

int modem_recv(int longueur_max, unsigned char *message, int *long_msg_recu,
               unsigned long *adresse_envoyeur)
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * modem_close: coupe la connexion par modem (raccroche)
 */

int modem_close()
{
    /*  A DEVELOPPER  */
    return 1;
}

/*
 * modem_nom2addr: transforme un nom en une adresse
 */

unsigned long modem_nom2addr(unsigned char *nom_machine)
{
    /*  A DEVELOPPER  */
    return 0;
}
